<?php
/**
 * Created by PhpStorm.
 * User: Vladislav
 * Date: 26.02.2016
 * Time: 11:56
 */

namespace FS\BasicBundle\Migration;


use Doctrine\ORM\EntityManager;
use FS\BasicBundle\Entity\Migration;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Utils\WowzaManager;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Entity\UserGroupRight;
use FS\UserBundle\Model\UserGroupSearchResult;

/**
 * Class MigrationHelper
 * @package FS\BasicBundle\Migration
 * @author <vladislav@fora-soft.com>
 */
class MigrationHelper
{
    /** @var  WowzaManager $wowzaManager */
    private $wowzaManager;
    /** @var  EntityManager $em */
    private $em;

    public function __construct(EntityManager $entityManager, WowzaManager $wowzaManager)
    {
        $this->em = $entityManager;
        $this->wowzaManager = $wowzaManager;
    }

    public function migrate()
    {
        $migrations = $this->em->getRepository("FSBasicBundle:Migration")->findAll();
        $needToMigrate = $this->getMigrationList();

        foreach ($migrations as $migration) {
            unset($needToMigrate[$migration->getMigrationName()]);
        }
        foreach ($needToMigrate as $migrate) {
            call_user_func([$this, $migrate]);
        }

        return count($needToMigrate);
    }

    private function getMigrationList()
    {
        return [
            Migration::MIGRATION__MULTI_CAMERAS => "camerasMigrate",
            Migration::MIGRATION__CONTAINERS => "containerMigrate",
            Migration::MIGRATION__ADMIN_GROUP_RIGHTS => "adminGroupRightsMigrate",
            Migration::MIGRATION__ADMIN_GROUP_SEARCH => "adminGroupSearchMigrate",
            Migration::MIGRATION__THUMBNAILS_VIEW_ADD_IMAGE => 'createThumbnailsMigrate',
            Migration::MIGRATION__SET_IO_USER => 'setIOUserMigrate',
            Migration::MIGRATION__ADMIN_UPDATE_RIGHT => 'adminUpdateRightMigrate'
        ];
    }

    private function camerasMigrate()
    {
        $cameras = $this->em->getRepository("FSCamBundle:Camera")->findAll();
        foreach ($cameras as $camera) {
            if ($camera->getRoomOld() != null) {
                $room = $camera->getRoomOld();
                $camera->addRoom($room);
                $camera->getRoomOld()->removeCameraOld($camera);
            }
            $camera->setRoomOld();

            $this->em->persist($camera);
            $this->em->flush();
        }
        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__MULTI_CAMERAS);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function containerMigrate()
    {
        $mainContainer = $this->em->getRepository("FSUserBundle:Container")->getContainerHead();
        if ($mainContainer == null) {
            $container = new Container();
            $container->setName('Global');
            $container->setParent(null);
            $this->em->persist($container);
            $this->em->flush();

            $userGroup = $this->em->getRepository('FSUserBundle:UserGroup')->findOneBy(['id' => 1]);
            $userGroup->setContainer($container);
            $this->em->persist($userGroup);
            $this->em->flush();
        }

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__CONTAINERS);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function adminGroupRightsMigrate()
    {
        $userGroup = $this->em->getRepository('FSUserBundle:UserGroup')->findOneBy(['id' => 1]);
        $container = $this->em->getRepository("FSUserBundle:Container")->getContainerHead();
        $userGroup->setContainer($container);

        $rightsList = UserGroupRight::getGruntsList();

        foreach ($rightsList as $right) {
            if (!$userGroup->hasRight($right)) {
                $newRight = new UserGroupRight();
                $newRight->setGranted($right);
                $userGroup->addRight($newRight);
                $this->em->persist($newRight);
                $this->em->flush();
            }
        }
        $this->em->persist($userGroup);
        $this->em->flush();

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__ADMIN_GROUP_RIGHTS);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function adminGroupSearchMigrate()
    {
        $userGroups = $this->em->getRepository('FSUserBundle:UserGroup')->getAllGroupsForChoice();

        /** @var UserGroup $userGroup */
        foreach ($userGroups as $userGroup) {
            $userGroup->setSearchResults(UserGroupSearchResult::getUserGroupSearchResultsList());
            $this->em->persist($userGroup);
        }

        $this->em->flush();

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__ADMIN_GROUP_SEARCH);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function createThumbnailsMigrate()
    {
        $records = $this->em->getRepository('FSCamBundle:Record')->findAll();
        foreach ($records as $record) {
            if (!$record->isRecordPreviewReady()) {
                if (
                    $record->getRecordType() == Record::RECORD_TYPE_RECORD
                    || $record->getRecordType() == null
                ) {
                    $this->wowzaManager->makeThumbnailForRecord($record);
                } else {
                    $this->wowzaManager->makeThumbnailLocalWowza($record);
                }
            }
        }

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__THUMBNAILS_VIEW_ADD_IMAGE);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function setIOUserMigrate() {
        $rooms = $this->em->getRepository('FSCamBundle:Room')->findAll();
        $groups = $this->em->getRepository('FSUserBundle:UserGroup')->findAll();
        foreach ($rooms as $room) {
            if ($room->getIsCameraIO() && $room->getUserIO() === null) {
                foreach ($groups as $group) {
                    if ($group->getRooms()->contains($room) && !$group->getUsers()->isEmpty()) {
                        $userIO = $group->getUsers()->first();
                        $room->setUserIO($userIO);

                        $this->em->persist($userIO);
                        $this->em->flush();
                        break;
                    }
                }
            }
        }

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__SET_IO_USER);

        $this->em->persist($migration);
        $this->em->flush();
    }

    private function adminUpdateRightMigrate() {
        $userGroup = $this->em->getRepository('FSUserBundle:UserGroup')->findOneBy(['id' => 1]);

        if (!$userGroup->hasRight(UserGroupRight::GRANT_ADMIN__UPDATE)) {
            $newRight = new UserGroupRight();
            $newRight->setGranted(UserGroupRight::GRANT_ADMIN__UPDATE);
            $userGroup->addRight($newRight);
            $this->em->persist($newRight);
            $this->em->flush();
        }

        $migration = new Migration();
        $migration->setMigrationName(Migration::MIGRATION__ADMIN_UPDATE_RIGHT);

        $this->em->persist($migration);
        $this->em->flush();
    }
}