<?php

namespace FS\CamBundle\Command;

use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Recur;
use FS\CamBundle\Entity\Schedule;
use FS\UserBundle\Entity\Log;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use FS\CamBundle\Entity\Recording;

class ScheduledRecorderCommand extends ContainerAwareCommand
{
    const SCHEDULE_UPDATE_REDIS_CHANNEL = 'schedule:update';
    const SCHEDULE_DELETE_REDIS_CHANNEL = 'schedule:delete';

    protected function configure()
    {
        $this
            ->setName('recorder:scheduled:run')
            ->setDescription('Run scheduled recorder');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        //date_default_timezone_set("UTC");
        /** @var \FS\CamBundle\Utils\Recorder $recorder */
        $recorder = $this->getContainer()->get('fs_cam.recorder');

        $now = new \DateTime('now');

        /** @var \Doctrine\ORM\EntityManager $em */
        $em = $this->getContainer()->get('doctrine')->getManager();

        /**
         * STOP EXISTING RECORDS
         */

        $recordings = $em->getRepository("FSCamBundle:Recording")->getExpiredRecordings($now);

        /** @var Recording $recording */
        foreach ($recordings as $recording) {
            $output->writeln('<info>Stop Record</info>');
            $output->writeln('<info>' . $recording->getId() . '</info>');
            $schedule = $recording->getSchedule();
            $schedule->setStatus(Schedule::STATUS_PENDING);

            $recur = $schedule->getRecur();
            if ($recur && $recur->getRange() == Recur::RANGE_END_BY_OCCURENCES && $recur->getOccurrences() == 0) {
                $em->remove($schedule);
            } else if (!$recur) {
                $em->remove($schedule);
            } else {
                $schedule->setRealStartAt($schedule->getNextStartAt());
                $em->persist($schedule);
            }

            $record = $recording->getRecord();
            $record->setIsCompleted();
            $recorder->stopRecording($record);
            $em->remove($recording);

            $recording->getSchedule()->setRecording(null);
            $recording->setSchedule(null);
            $recording->setRoom(null);
            $em->remove($recording);

            $extraFields = function(Log $log) {
                $log->setExtraInformation('Stop schedule');
            };

            $this->getContainer()->get('fs_user.util.logger')->logAction(
                Log::OB_TYPE_RECORDING,
                $record,
                $schedule->getCreatedBy(),
                Log::$actions[Log::OB_TYPE_RECORDING]['scheduleStop'],
                $extraFields);
        }
        $em->flush();


        /**
         * CHANGE STATUS OF EXPIRED WITHOUT RECUR SCHEDULES
         */
        $expiredSchedules = $em->createQuery(
            'SELECT s, r FROM FSCamBundle:Schedule s LEFT JOIN s.recur r
            WHERE r.id IS NULL
            AND s.status = :status
            AND (UNIX_TIMESTAMP(s.startAt) + s.duration) <= :now'
        )
            ->setParameter('now', $now->getTimestamp())
            ->setParameter('status', Schedule::STATUS_PENDING)
            ->getResult();

        /** @var Schedule $schedule */
        foreach ($expiredSchedules as $schedule) {
            if (($schedule->getStartAt()->getTimestamp() + $schedule->getDuration()) <= $now->getTimeStamp()) {
                $em->remove($schedule);
            }
        }
        $em->flush();

        /**
         * CHANGE STATUS OF EXPIRED BY DATE SCHEDULES
         */
        $expiredSchedules = $em->createQuery(
            'SELECT s FROM FSCamBundle:Schedule s JOIN s.recur r
            WHERE r.endAt <= :now AND r.range = :end_by_date'
        )
            ->setParameter('now', $now)
            ->setParameter('end_by_date', Recur::RANGE_END_BY_DATE)
            ->getResult();

        /** @var Schedule $schedule */
        foreach ($expiredSchedules as $schedule) {
            $output->writeln('<info>I here 220</info>');
            $em->remove($schedule);
        }
        $em->flush();

        /**
         * CHECK FOR EXCEPTIONS
         */

        // if exception, we do nothing
        $exceptions = $em->createQuery(
            'SELECT e FROM FSCamBundle:Exception e WHERE e.date LIKE :now'
        )
            ->setParameter('now', $now->format('Y-m-d') . '%')
            ->getResult();

        if ($exceptions) {
            $output->writeln('<info>Exception.</info>');
            $schedules = [];
        } else {
            $schedules = $em->createQuery(
                'SELECT s FROM FSCamBundle:Schedule s WHERE s.startAt <= :now AND s.status = :status
                AND s.room NOT IN(
                SELECT room FROM FSCamBundle:Room room LEFT JOIN room.recording recording WHERE recording.room = room.id
                )'
            )
                ->setParameter('now', $now)
                ->setParameter('status', Schedule::STATUS_PENDING)
                ->getResult();
        }

        $scheduledToStart = [];
        $scheduledRoomIds = [];

        /** @var \FS\CamBundle\Entity\Schedule $schedule */
        foreach ($schedules as $schedule) {
            $output->writeln('<info>Searching</info>');
            $recur = $schedule->getRecur();
            $output->writeln('<info>' . $schedule->getStartAt()->format('d-m-Y H:i') . '</info>');
            $output->writeln('<info>' . $now->format('d-m-Y H:i') . '</info>');

            // if schedule ends by date, and date expired change schedule status to expired.
            if ($recur && $recur->getRange() == Recur::RANGE_END_BY_DATE && $now > $recur->getEndAt()) {
                $em->remove($schedule);
                continue;
            }

            // always compare time
            if (!$schedule->timeInRange($now)) {
                continue;
            }

            if ($recur) {
                if ($schedule->getRealStartAt() !== null) {
                    $output->writeln('<info>' . $schedule->getRealStartAt()->format('d-m-Y H:i') . '</info>');
                    if (strtotime($schedule->getRealStartAt()->format('d-m-Y')) > strtotime($now->format('d-m-Y'))) {
                        continue;
                    } else {
                        switch ($recur->getFrequency()) {
                            case Recur::FREQ_DAILY: // no needs to compare anything but time
                                break;
                            case Recur::FREQ_MONTHLY: // compare day of month
                                if ($now->format('d') != $schedule->getStartAt()->format('d')) {
                                    continue 2;
                                }
                                break;
                            case Recur::FREQ_WEEKLY: // check existence weekday in weekdays list
                                if (!in_array($now->format('w'), $recur->getWeekly())) {
                                    continue 2;
                                }
                                break;
                        }
                    }
                }
            } else { // no recur
                // compare dates
                if ($schedule->getStartAt()->format('d-m-Y') != $now->format('d-m-Y')) {
                    continue;
                }
            }
            // reduce duplicates by roomId
            $roomId = $schedule->getRoom()->getId();
            if (!in_array($roomId, $scheduledRoomIds)) {
                $output->writeln('<info>add ' . $schedule->getId() . '</info>');
                $scheduledToStart[] = $schedule;
                $scheduledRoomIds[] = $roomId;
            }
        }

        $recordsToStart = [];

        /** @var \FS\CamBundle\Entity\Schedule $schedule */
        foreach ($scheduledToStart as $schedule) {
            $record = new Record();
            $record->setStartedAt($now);
           // $record->setNameTitle($schedule->getNameTitle());
            $record->setName($schedule->getName());
            $record->setRoom($schedule->getRoom());
            $record->setAccessedByGroups($schedule->getAccessedByGroups());
            $record->setAccessedByUsers($schedule->getAccessedByUsers())
                ->setContainer($schedule->getContainer());


            if ($schedule->getTemplate() !== null) {
                $record->setTemplateData($schedule->getTemplateData());
                $record->setTemplate($schedule->getTemplate());
            }

            $record->setParentRetentionRule($schedule->getRetentionRule());
            $record->setCreatedBy($schedule->getCreatedBy());

            $schedule->setStatus(Schedule::STATUS_STARTED);
            $em->persist($schedule);
            $em->persist($record);


            $recordsToStart[] = $record;

            $recording = new Recording();
            $recording->setStartedAt($now);
            $recording->setSchedule($schedule);
            $recording->setRecord($record);
            $stopAt = clone $schedule->getStopAtTime();
            $stopAt->setDate($now->format('Y'), $now->format('m'), $now->format('d'));
            $recording->setStopAt($stopAt);
            $em->persist($recording);
            $em->flush();

            // if schedule ended, change status to expired
            $recur = $schedule->getRecur();
            if ($recur && $recur->getRange() == Recur::RANGE_END_BY_OCCURENCES) {
                $output->writeln('<info>I here 220</info>');
                $recur->setOccurrences($recur->getOccurrences() - 1);
            }


            $startResult = $recorder->startRecording($record, $schedule->getPresets()->toArray());
            if (!$startResult) {
                $schedule = $recording->getSchedule();
                $schedule->setStatus(Schedule::STATUS_PENDING);
                $em->persist($schedule);
                $em->remove($recording);
                $em->remove($record);
                $em->flush();
            }else {
                $extraFields = function(Log $log) use ($schedule) {
                    $log->setContainer($schedule->getContainer())
                        ->setExtraInformation('Start schedule');
                };

                $this->getContainer()->get('fs_user.util.logger')->logAction(
                    Log::OB_TYPE_RECORDING,
                    $record,
                    $schedule->getCreatedBy(),
                    Log::$actions[Log::OB_TYPE_RECORDING]['scheduleStart'],
                    $extraFields);
            }
        }
        $em->flush();


        ////Дикий костыль
        $recordings = $em->getRepository("FSCamBundle:Recording")->getExpiredRecordingsToDel($now);
        foreach ($recordings as $recording) {
            $output->writeln('<info>I here 256-258</info>');
            if ($recording->getSchedule() == null)
                $em->remove($recording);
        }
        $em->flush();


        /* Finding for blocked schedule */

        $schedules = $em->createQuery(
            'SELECT s FROM FSCamBundle:Schedule s WHERE s.startAt <= :now AND s.status = :status
                AND s.room IN(
                SELECT room FROM FSCamBundle:Room room LEFT JOIN room.recording recording WHERE recording.room = room.id
                )'
        )
            ->setParameter('now', $now)
            ->setParameter('status', Schedule::STATUS_PENDING)
            ->getResult();

        $blocks = [];
        /** @var Schedule $blockedSchedule */
        foreach ($schedules as $blockedSchedule) {
            if ($blockedSchedule->getRealStartAt() != null) {
                if ($blockedSchedule->getRealStartAt()->format('d-m-Y') != $now->format('d-m-Y')) {
                    $this->sendStatus($blockedSchedule->getId(), 'ready');
                    continue;
                }
                $stop = clone $blockedSchedule->getRealStartAt();
                $stop->modify('+ ' . $blockedSchedule->getDuration() . ' seconds');
                if (
                    !($blockedSchedule->getRealStartAt()->format('d-m-Y H:i') < $now->format('d-m-Y H:i'))
                    &&
                    !($stop->format('d-m-Y H:i') < $now->format('d-m-Y H:i'))
                ) {
                    $this->sendStatus($blockedSchedule->getId(), 'ready');
                    continue;
                }
            } else {
                if ($blockedSchedule->getStartAt()->format('d-m-Y') != $now->format('d-m-Y')) {
                    $this->sendStatus($blockedSchedule->getId(), 'ready');
                    continue;
                }
                if (!$blockedSchedule->timeInRange($now)) {
                    $this->sendStatus($blockedSchedule->getId(), 'ready');
                    continue;
                }
            }
            $blocks[] = $blockedSchedule;

            $extraFields = function(Log $log) use ($schedule) {
                $log->setContainer($schedule->getContainer())
                    ->setExtraInformation('Blocked schedule');
            };

            $this->getContainer()->get('fs_user.util.logger')->logAction(
                Log::OB_TYPE_RECORDING,
                $blockedSchedule,
                $schedule->getCreatedBy(),
                Log::$actions[Log::OB_TYPE_RECORDING]['scheduleBlock'],
                $extraFields);

            $this->sendStatus($blockedSchedule->getId(), 'blocked');
        }

        $schedules = $em->createQuery(
            'SELECT s FROM FSCamBundle:Schedule s WHERE s.startAt < :now'
        )
            ->setParameter('now', $now)
            ->getResult();

        /** @var Schedule $futureSchedule */
        $conflictedSchedules = [];
        foreach ($schedules as $futureSchedule) {
            $conflict = $futureSchedule->isBlocked($schedules);
            if ($conflict) {
                $conflictedSchedules[] = $conflict;
                $conflictedSchedules[] = $futureSchedule;
            } else {
                //$this->sendStatus($futureSchedule->getId(), 'ready');
            }
            foreach ($conflictedSchedules as $schedule) {
                if (
                    $schedule->getStatus() == Schedule::STATUS_PENDING
                    &&
                    !in_array($schedule, $blocks)
                ) {
                    $this->sendStatus($schedule->getId(), 'conflict');
                }
            }
        }
        $output->writeln('<info>Done!</info>');
    }

    private function sendStatus($id, $status)
    {
        $this->getContainer()->get('snc_redis.default')->publish(
            self::SCHEDULE_UPDATE_REDIS_CHANNEL,
            json_encode(
                [
                    'id' => $id,
                    'status' => $status,
                ]
            )
        );
    }
}
