<?php

namespace FS\CamBundle\Entity;

use FS\BasicBundle\Model\Loggable;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsInterface;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsTrait;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Model\ContainerizedInterface;
use Gedmo\Mapping\Annotation as Gedmo;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Validator\Constraints as Assert;
use FS\UserBundle\Entity\User;
use FS\CamBundle\Template\TemplateDataTrait;
use FS\CamBundle\Template\TemplateAwareTrait;
use FS\CamBundle\Template\TemplateAwareInterface;

/**
 * Record
 *
 * @ORM\Table(name="records")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\RecordRepository")
 */
class Record implements
    TemplateAwareInterface,
    AccessedByUsersAndGroupsInterface,
    Loggable,
    ContainerizedInterface
{
    use TemplateDataTrait;
    use TemplateAwareTrait;

    use AccessedByUsersAndGroupsTrait;

    use FSContainerTrait;


    const STATUS_COMPLETED = 1;
    const STATUS_UNCOMPLETED = 0;
    const STATUS_PARTIALLY_UPLOADED = 2;

    const RECORD_TYPE_RECORD = "record";
    const RECORD_TYPE_UPLOAD = "upload";
    const RECORD_TYPE_DEVICE = "device";

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank
     */
    private $name;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true)
     */
    private $duration;

    /**
     * @var Room
     *
     * @ORM\ManyToOne(targetEntity="Room", inversedBy="records")
     * @ORM\JoinColumn(name="room_id", referencedColumnName="id", onDelete="SET NULL")
     */
    private $room;

    /**
     * @var \DateTime
     *
     * @Gedmo\Timestampable(on="create")
     * @ORM\Column(type="datetime")
     */
    private $startedAt;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Marker", mappedBy="record", cascade={"persist", "remove"})
     */
    private $markers;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     */
    private $status;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $recordType;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $recordUploadetSequence;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    private $recordShareSequence;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $shared;

    /**
     * @var User
     * @Gedmo\Blameable(on="create")
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\User")
     * @ORM\JoinColumn(name="created_by", referencedColumnName="id", onDelete="SET NULL")
     */
    private $createdBy;

    /**
     * @var RetentionRule
     *
     * @ORM\ManyToOne(targetEntity="RetentionRule",cascade={"persist"})
     * @ORM\JoinColumn(referencedColumnName="id", nullable=true)
     */
    private $retentionRule;

    /**
     * @var RetentionRule
     *
     * @ORM\ManyToOne(targetEntity="RetentionRule",cascade={"persist"})
     * @ORM\JoinColumn(referencedColumnName="id")
     */
    private $parentRetentionRule;

    /**
     * @var Object
     *
     * @ORM\Column(type="json_array", nullable=true)
     */
    private $camerasInRecord;

    /**
     * @var MosaicQueue
     * @ORM\OneToOne(targetEntity="FS\CamBundle\Entity\MosaicQueue", mappedBy="record", cascade={"persist", "remove"})
     */
    private $mosaic;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", options={"default":false})
     */
    private $recordPreviewReady;

    /**
     * Constructor
     */
    public function __construct()
    {
        //Default type is record
        $this->recordType = self::RECORD_TYPE_RECORD;
        $this->status = self::STATUS_UNCOMPLETED;
        $this->accessedByUsers = new ArrayCollection();
        $this->accessedByGroups = new ArrayCollection();
        $this->recordPreviewReady = false;
    }

    /*public function __clone()
    {
        if ($this->id) {
            $this->setId(null);
        }
    }*/

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set id
     *
     * @return int
     */
    protected function setId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Record
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set room
     *
     * @param  \FS\CamBundle\Entity\Room $room
     * @return Camera
     */
    public function setRoom(\FS\CamBundle\Entity\Room $room = null)
    {
        $this->room = $room;
        if($room != null ){
            $cameras = $this->getRoom()->camerasToArrayKeyVal();
            $this->setCamerasInRecord($cameras);
        }
        return $this;
    }

    /**
     * Get room
     *
     * @return \FS\CamBundle\Entity\Room
     */
    public function getRoom()
    {
        return $this->room;
    }

    /**
     * Set startedAt
     *
     * @param  \DateTime $startedAt
     * @return Record
     */
    public function setStartedAt($startedAt)
    {
        $this->startedAt = $startedAt;

        return $this;
    }

    /**
     * Get startedAt
     *
     * @return \DateTime
     */
    public function getStartedAt()
    {
        return $this->startedAt;
    }

    /**
     * Add cameras
     *
     * @param  \FS\CamBundle\Entity\Marker $marker
     * @return Room
     */
    public function addMarker(\FS\CamBundle\Entity\Marker $marker)
    {
        $this->markers[] = $marker;

        return $this;
    }

    /**
     * Remove cameras
     *
     * @param \FS\CamBundle\Entity\Marker $marker
     */
    public function removeRecord(\FS\CamBundle\Entity\Marker $marker)
    {
        $this->markers->removeElement($marker);
    }

    /**
     * Get cameras
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getMarkers()
    {
        return $this->markers;
    }

    /**
     * @return boolean
     */
    public function isCompleted()
    {
        return $this->status == self::STATUS_COMPLETED;
    }

    public function setIsCompleted()
    {
        $this->status = self::STATUS_COMPLETED;
    }

    public function setIsUncompleted()
    {
        $this->status = self::STATUS_UNCOMPLETED;
    }

    public function setIsPartiallyUpload()
    {
        $this->status = self::STATUS_PARTIALLY_UPLOADED;
    }

    /**
     * @return User
     */
    public function getCreatedBy()
    {
        return $this->createdBy;
    }

    /**
     * @param User $createdBy
     * @return $this
     */
    public function setCreatedBy(User $createdBy)
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    /**
     * Set retentionRule
     *
     * @param RetentionRule $retentionRule
     * @return Record
     */
    public function setRetentionRule($retentionRule)
    {
        $this->retentionRule = $retentionRule;

        return $this;
    }

    /**
     * Get retentionRule
     *
     * @return RetentionRule
     */
    public function getRetentionRule()
    {
        return $this->retentionRule;
    }

    /**
     * Set parentRetentionRule
     *
     * @param RetentionRule $parentRetentionRule
     * @return Record
     */
    public function setParentRetentionRule($parentRetentionRule)
    {
        $this->parentRetentionRule = $parentRetentionRule;

        return $this;
    }

    /**
     * Get parentRetentionRule
     *
     * @return RetentionRule
     */
    public function getParentRetentionRule()
    {
        return $this->parentRetentionRule;
    }

    /**
     * @return int
     */
    public function getDuration()
    {
        return $this->duration;
    }

    /**
     * @param $duration
     */
    public function setDuration($duration)
    {
        $this->duration = $duration;
        return $this;
    }

    /**
     * @param $recordType
     * @return $this
     */
    public function setRecordType($recordType)
    {
        $this->recordType = $recordType;

        return $this;
    }

    /**
     * @return string
     */
    public function getRecordType()
    {
        return $this->recordType;
    }

    /**
     * @param $recordUploadetSequence
     * @return $this
     */
    public function setRecordUploadetSequence($recordUploadetSequence)
    {
        $this->recordUploadetSequence = $recordUploadetSequence;
        return $this;
    }

    /**
     * @return string
     */
    public function getRecordUploadetSequence()
    {
        return $this->recordUploadetSequence;
    }

    /**
     * @param $recordShareSequence
     * @return $this
     */
    public function setRecordShareSequence($recordShareSequence)
    {
        $this->recordShareSequence = $recordShareSequence;

        return $this;
    }

    /**
     * @return string
     */
    public function getRecordShareSequence()
    {
        return $this->recordShareSequence;
    }

    /**
     * @param $shared
     * @return $this
     */
    public function setShared($shared)
    {
        $this->shared = $shared;
        return $this;
    }

    /**
     * @return bool
     */
    public function getShared()
    {
        return $this->shared;
    }

    /**
     * @return MosaicQueue
     */
    public function getMosaic()
    {
        return $this->mosaic;
    }

    /**
     * @param $mosaicQueue
     */
    public function setMosaic($mosaicQueue)
    {
        $this->mosaic = $mosaicQueue;
    }

    public function getDurationFormatted()
    {
        $hours = floor($this->duration / 3600);
        $minutes = floor(($this->duration / 60) % 60);
        $seconds = $this->duration % 60;

        return "$hours:$minutes:$seconds";
    }

    public function getObjectName()
    {
        return $this->name;
    }

    /**
     * @return array
     */
    public function getCamerasInRecord()
    {
        if($this->camerasInRecord == null){
            if($this->room != null){
                return $this->room->camerasToArrayKeyVal();
            }
        }
        return $this->camerasInRecord;
    }

    public function getOnlyCamerasInRecord()
    {
        return $this->camerasInRecord;
    }

    /**
     * @param $camerasInRecord
     * @return $this
     */
    public function setCamerasInRecord($camerasInRecord)
    {
        $this->camerasInRecord = $camerasInRecord;

        return $this;
    }

    public function getRoomName()
    {
        if ($this->room == null) {
            return 'No room';
        } else {
            return $this->room->getName();
        }
    }

    /**
     * @return boolean
     */
    public function isRecordPreviewReady()
    {
        return $this->recordPreviewReady;
    }

    /**
     * @param boolean $recordPreviewReady
     * @return $this
     */
    public function setRecordPreviewReady($recordPreviewReady)
    {
        $this->recordPreviewReady = $recordPreviewReady;
        return $this;
    }

}
