<?php

namespace FS\CamBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use FS\BasicBundle\Model\Loggable;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use FS\UserBundle\Model\ContainerizedInterface;
use Symfony\Component\Validator\Constraints as Assert;


/**
 * Room
 *
 * @ORM\Table(name="rooms")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\RoomRepository")
 */
class Room implements Loggable, ContainerizedInterface
{

    use FSContainerTrait;

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=128)
     * @Assert\NotBlank()
     */
    private $name;

    /**
     * @var ArrayCollection
     * @ORM\ManyToMany(targetEntity="Camera", mappedBy="rooms", cascade={"persist", "remove"})
     */
    private $cameras;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Camera", mappedBy="roomOld")
     */
    private $camerasOld;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Record", mappedBy="room", cascade={"persist"})
     */
    private $records;

    /**
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Schedule", mappedBy="room", cascade={"persist", "remove"})
     */
    private $schedules;

    /**
     * @var Recording
     *
     * @ORM\OneToOne(targetEntity="Recording", mappedBy="room", cascade={"persist", "remove"})
     */
    private $recording;

    /**
     * @var User
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\User", mappedBy="sharedRooms")
     */
    private $users;

    /**
     * @var ArrayCollection | \FS\UserBundle\Entity\UserGroup[]
     * @ORM\ManyToMany(targetEntity="FS\UserBundle\Entity\UserGroup")
     **/
    private $groups;

    /**
     * @var User
     * @ORM\ManyToMany(targetEntity="FS\CamBundle\Entity\Filter", mappedBy="filterRooms")
     */
    private $filters;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", nullable=true)
     */
    private $isCameraIO;

    /**
     * @var Camera
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\Camera")
     * @ORM\JoinColumn(name="camera_io_id", referencedColumnName="id", onDelete="SET NULL")
     */
    private $cameraIO;

    /**
     * @var User
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\User")
     * @ORM\JoinColumn(name="user_io_id", referencedColumnName="id", onDelete="SET NULL")
     */
    private $userIO;

    public function __construct()
    {
        $this->cameras = new ArrayCollection();
        $this->camerasOld = new ArrayCollection();
        $this->users = new ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Room
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Add cameras
     *
     * @param  \FS\CamBundle\Entity\Camera $cameras
     * @return Room
     */
    public function addCamera(\FS\CamBundle\Entity\Camera $cameras)
    {
        $this->cameras[] = $cameras;

        return $this;
    }

    /**
     * Remove cameras
     *
     * @param \FS\CamBundle\Entity\Camera $cameras
     */
    public function removeCamera(\FS\CamBundle\Entity\Camera $cameras)
    {
        $this->cameras->removeElement($cameras);
    }

    /**
     * Get cameras
     *
     * @return \Doctrine\Common\Collections\Collection | Camera[]
     */
    public function getCameras()
    {
        return $this->cameras;
    }

    /**
     * Get camerasOld
     *
     * @return \Doctrine\Common\Collections\Collection | Camera[]
     */
    public function getCamerasOld()
    {
        return $this->camerasOld;
    }

    /**
     * Remove cameras
     *
     * @param \FS\CamBundle\Entity\Camera $cameras
     */
    public function removeCameraOld(\FS\CamBundle\Entity\Camera $cameras)
    {
        $this->camerasOld->removeElement($cameras);
    }

    /**
     * Add cameras
     *
     * @param  \FS\CamBundle\Entity\Record $record
     * @return Room
     */
    public function addRecord(\FS\CamBundle\Entity\Record $record)
    {
        $this->records[] = $record;

        return $this;
    }

    /**
     * Remove cameras
     *
     * @param \FS\CamBundle\Entity\Record $cameras
     */
    public function removeRecord(\FS\CamBundle\Entity\Record $record)
    {
        $this->records->removeElement($record);
    }

    /**
     * Get cameras
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getRecords()
    {
        return $this->records;
    }

    /**
     * @param \FS\CamBundle\Entity\Recording $recording
     * @return $this
     */
    public function setRecording($recording)
    {
        $this->recording = $recording;

        return $this;
    }

    /**
     * @return \FS\CamBundle\Entity\Recording
     */
    public function getRecording()
    {
        return $this->recording;
    }

    public function hasRecording()
    {
        return $this->getRecording() !== null;
    }


    /**
     * @param User $users
     * @return $this
     */
    public function addUsers(\FS\UserBundle\Entity\User $users)
    {
        $this->users[] = $users;

        return $this;
    }

    /**
     * @param User $users
     */
    public function removeUsers(\FS\UserBundle\Entity\User $users)
    {
        $this->users->removeElement($users);
    }

    /**
     * @return ArrayCollection|User
     */
    public function getUsers()
    {
        return $this->users;
    }

    /**
     * @return ArrayCollection|\FS\UserBundle\Entity\UserGroup[]
     */
    public function getGroups()
    {
        return $this->groups;
    }

    /**
     * @param UserGroup $group
     * @return $this
     */
    public function addGroup(UserGroup $group)
    {
        $this->groups[] = $group;

        return $this;
    }

    /**
     * @param UserGroup $group
     * @return $this
     */
    public function removeGroup(UserGroup $group)
    {
        $this->groups->removeElement($group);

        return $this;
    }

    /**
     * @param \FS\CamBundle\Entity\Camera $camera
     * @return $this
     */
    public function setCameraIO($camera)
    {
        $this->cameraIO = $camera;

        return $this;
    }

    /**
     * @return \FS\CamBundle\Entity\Camera
     */
    public function getCameraIO()
    {
        return $this->cameraIO;
    }

    /**
     * @param \FS\UserBundle\Entity\User $user
     * @return $this
     */
    public function setUserIO($user)
    {
        $this->userIO = $user;

        return $this;
    }

    /**
     * @return \FS\UserBundle\Entity\User
     */
    public function getUserIO()
    {
        return $this->userIO;
    }

    /**
     * Set name
     *
     * @param $isCameraIO
     * @return Room
     *
     */
    public function setIsCameraIO($isCameraIO)
    {
        $this->isCameraIO = $isCameraIO;

        return $this;
    }

    /**
     * Get name
     *
     * @return boolean
     */
    public function getIsCameraIO()
    {
        return $this->isCameraIO;
    }

    public function getWowzas()
    {
        $cameras = $this->getCameras();

        $result = [];
        foreach ($cameras as $camera) {
            if (!in_array($camera->getWowza(), $result)) {
                $result[] = $camera->getWowza();
            }
        }

        return $result;
    }

    public function __toString()
    {
        return $this->name;
    }

    public function getObjectName()
    {
        return $this->name;
    }

    /**
     * Remove filters
     *
     * @param \FS\CamBundle\Entity\Filter $filters
     */
    public function removeFilter(\FS\CamBundle\Entity\Filter $filters)
    {
        $this->filters->removeElement($filters);
    }

    /**
     * Get filters
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilters()
    {
        return $this->filters;
    }

    /**
     * @return int
     */
    public function numCamerasInRoom()
    {
        return $this->getCameras()->filter(function (Camera $camera) {
            return $camera->getDeviceType() === Camera::TYPE_CAMERA;
        })->count();

    }

    /**
     * @return array
     */
    public function camerasToArrayKeyVal()
    {
        $result = [];
        /** @var Camera $camera */
        foreach ($this->cameras as $camera) {
            if ($camera->getDeviceType() == Camera::DEVICE_TYPE_CARD_READER) {
                continue;
            }
            $result[$camera->getId()] = $camera->getName();

        }
        return $result;
    }
}
