<?php

namespace FS\CamBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use FS\BasicBundle\Model\Loggable;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Model\ContainerizedInterface;
use Gedmo\Mapping\Annotation as Gedmo;
use Doctrine\ORM\Mapping as ORM;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsInterface;
use FS\CamBundle\RecordAccess\AccessedByUsersAndGroupsTrait;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\ExecutionContextInterface;
use FS\CamBundle\Template\TemplateDataTrait;
use FS\CamBundle\Template\TemplateAwareTrait;
use FS\CamBundle\Template\TemplateAwareInterface;
use FS\Userbundle\Entity\User;

/**
 * Schedule
 *
 * @ORM\Table(name="schedules")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\ScheduleRepository")
 * @ORM\HasLifecycleCallbacks()
 */
class Schedule implements TemplateAwareInterface, AccessedByUsersAndGroupsInterface, Loggable, ContainerizedInterface
{
    use TemplateDataTrait;
    use TemplateAwareTrait;

    use AccessedByUsersAndGroupsTrait;

    use FSContainerTrait;

    const STATUS_PENDING = 0;
    const STATUS_STARTED = 1;
    const STATUS_EXPIRED = 2;

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank
     */
    private $name;

    /**
     * @var Room
     *
     * @ORM\ManyToOne(targetEntity="Room", inversedBy="schedules")
     * @ORM\JoinColumn(name="room_id", referencedColumnName="id")
     * @Assert\NotBlank
     */
    private $room;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime")
     */
    private $startAt;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $nextStartAt;

    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $realStartAt;

    /**
     * @var RetentionRule
     *
     * @ORM\ManyToOne(targetEntity="RetentionRule",cascade={"persist"})
     * @ORM\JoinColumn(referencedColumnName="id")
     */
    private $retentionRule;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @Assert\NotBlank
     */
    private $duration;


    /**
     * @var \DateTime
     *
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $doNotStartUntil;
    /**
     * @var Recording
     *
     * @ORM\OneToOne(targetEntity="Recording", mappedBy="schedule",cascade={"all"})
     */
    private $recording;

    /**
     * @var integer
     *
     * @ORM\Column(type="smallint")
     */
    private $status = self::STATUS_PENDING;

    /**
     * @var \FS\CamBundle\Entity\Recur
     * @ORM\OneToOne(targetEntity="Recur", cascade={"all"}, orphanRemoval=true)
     * @ORM\JoinColumn(name="recur_id", referencedColumnName="id")
     **/
    private $recur;

    /**
     * @ORM\ManyToMany(targetEntity="Preset")
     * @ORM\JoinTable(name="schedule_presets",
     *      joinColumns={@ORM\JoinColumn(name="schedule_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="preset_id", referencedColumnName="id", onDelete="CASCADE")}
     * )
     */
    private $presets;

    /**
     * @var User
     * @Gedmo\Blameable(on="create")
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\User")
     * @ORM\JoinColumn(name="created_by", referencedColumnName="id", onDelete="SET NULL")
     */
    private $createdBy;


    public function __construct()
    {
        $this->presets = new ArrayCollection();
        $this->accessedByUsers = new ArrayCollection();
        $this->accessedByGroups = new ArrayCollection();
        $this->accessedByUsersAndGroups = new ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Schedule
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set room
     *
     * @param  \FS\CamBundle\Entity\Room $room
     * @return Schedule
     */
    public function setRoom(\FS\CamBundle\Entity\Room $room = null)
    {
        $this->room = $room;

        return $this;
    }

    /**
     * Get room
     *
     * @return \FS\CamBundle\Entity\Room
     */
    public function getRoom()
    {
        return $this->room;
    }

    /**
     * Set startAt
     *
     * @param  \DateTime $startAt
     * @return Schedule
     */
    public function setStartAt($startAt)
    {
        $this->startAt = $startAt;

        return $this;
    }

    /**
     * Get startAt
     *
     * @return \DateTime
     */
    public function getStartAt()
    {
        return $this->startAt;
    }

    /**
     * Set realStartAt
     *
     * @param $realStartAt
     * @return Schedule
     */
    public function setRealStartAt($realStartAt)
    {
        $this->realStartAt = $realStartAt;

        return $this;
    }

    /**
     * Get startAt
     *
     * @return \DateTime
     */
    public function getRealStartAt()
    {
        return $this->realStartAt;
    }

    /**
     * @param int $duration
     */
    public function setDuration($duration)
    {
        $this->duration = $duration;
    }

    /**
     * @return int
     */
    public function getDuration()
    {
        return $this->duration;
    }

    /**
     * @return \DateTime
     */
    public function getDoNotStartUntil()
    {
        return (null === $this->doNotStartUntil) ? new \DateTime : $this->doNotStartUntil;
    }

    /**
     * @param \DateTime $doNotStartUntil
     */
    public function setDoNotStartUntil(\DateTime $doNotStartUntil)
    {
        $this->doNotStartUntil = $doNotStartUntil;

        return $this;
    }

    /**
     * Set status
     *
     * @param  integer $status
     * @return Schedule
     */
    public function setStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * Get status
     *
     * @return integer
     */
    public function getStatus()
    {
        return $this->status;
    }

    public function getStatusList()
    {
        return [
            self::STATUS_STARTED,
            self::STATUS_PENDING,
        ];
    }

    /**
     * Set recur
     *
     * @param  \FS\CamBundle\Entity\Recur $recur
     * @return Schedule
     */
    public function setRecur(\FS\CamBundle\Entity\Recur $recur = null)
    {
        $this->recur = $recur;

        return $this;
    }

    /**
     * Get recur
     *
     * @return \FS\CamBundle\Entity\Recur
     */
    public function getRecur()
    {
        return $this->recur;
    }

    public function isStartAtValid()
    {
        if ($this->getRecur()) {
            return true;
        }
        return $this->getStartAt() > new \DateTime();
    }

    public function isEndAtValid()
    {

        if (!$this->getRecur()) {
            return true;
        }
        if ($this->getRecur()->getRange() !== Recur::RANGE_END_BY_DATE) {
            return true;
        }

        if($this->getStartAt() < new \DateTime() ){
            $end = clone  $this->getRecur()->getEndAt();
            $start = clone $this->getNextStartAt();
            return $start < $end->modify('tomorrow')->modify('-1 second');
        }
        return $this->getStartAt() < $this->getRecur()->getEndAt();
    }

    /**
     * @Assert\Callback
     */
    public function validate(ExecutionContextInterface $context)
    {
        if (!$this->isStartAtValid()) {
            $context->addViolationAt(
                'startAt',
                'The Start time must be greater than now.',
                array(),
                null
            );
        }

        if (!$this->isEndAtValid()) {
            $context->addViolationAt(
                null, // 'recur.endAt',
                'The End by date must be greater than Now or Next start time.',
                array(),
                null
            );
        }
    }

    public function getEndAt()
    {
        $format = 'M j, g:i A';
        $string = '';
        if (!$this->getRecur()) {
            $startAt = clone $this->getStartAt();
            $string = $startAt
                ->modify('+ ' . $this->getDuration() . ' seconds')
                ->format($format);
        } else {
            $recur = $this->getRecur();
            switch ($recur->getRange()) {
                case Recur::RANGE_END_BY_OCCURENCES:
                    $string = 'After ' . $recur->getOccurrences() . ' occur.';
                    break;
                case Recur::RANGE_NO_END_DATE:
                    $string = 'No end date';
                    break;
                case Recur::RANGE_END_BY_DATE:
                    $string = $recur->getEndAt()->format($format);
                    break;
            }
        }

        return $string;
    }

    public function getNextStartAt()
    {
        $nextStartAt = $this->nextStartAt;
        if ($nextStartAt < $this->getDoNotStartUntil()) {
            $nextStartAt = $this->calculateNextStartAt();
        }
        $this->setNextStartAt($nextStartAt);

        return $nextStartAt;
    }

    public function getJustNextStart()
    {
        return $this->nextStartAt;
    }

    public function setNextStartAt($nextStart)
    {
        $this->nextStartAt = $nextStart;

        return $this;
    }

    public function getNextStopAt()
    {
        $nextStartAt = clone $this->getNextStartAt();

        return $nextStartAt->modify('+ ' . $this->getDuration() . ' seconds');
    }


    public function calculateNextStartAt()
    {
        $now = new \DateTime;
        $doNotStartUntil = ($this->doNotStartUntil > $now) ? $this->doNotStartUntil : $now;
        if ($this->getStartAt() > $doNotStartUntil) {
            return $this->getStartAt();
        }
        $recur = $this->getRecur();

        if (!$this->nextStartAt) {
            $nextStartAt = clone $this->getStartAt();
        } else {
            $nextStartAt = $this->nextStartAt;
        }

        if (!$recur) {
            return $nextStartAt;
        }

        do {
            $this->calculateRecurNextStartAt($nextStartAt, $recur);
        } while ($nextStartAt < $doNotStartUntil);

        return $nextStartAt;

    }

    public function calculateRecurNextStartAt(\DateTime $nextStartAt, Recur $recur)
    {

        switch ($recur->getFrequency()) {
            case Recur::FREQ_MONTHLY:
                $nextStartAt->modify('+1 month');
                break;
            case Recur::FREQ_DAILY:
                $nextStartAt->modify('+1 day');
                break;
            case Recur::FREQ_WEEKLY:
                $weekDay = $nextStartAt->format('w');
                $weekDays = $recur->getWeekly();

                // find next weekday in weekly list
                do {
                    if ($weekDay < 6) {
                        $weekDay++;
                    } else {
                        $weekDay = 0;
                    }
                } while (!in_array($weekDay, $weekDays));

                $weekDayList = $recur::getWeekList();
                $weekDayName = $weekDayList[$weekDay];
                $startAt = clone $this->getStartAt();
                $nextStartAt->modify('next ' . $weekDayName);
                // after modify('next ...') time resets to 00:00, and we need set it again;
                $nextStartAt->setTime($startAt->format('H'), $startAt->format('i'), 0);

                break;
        }
        return $nextStartAt;
    }

    /**
     * Add presets
     *
     * @param  \FS\CamBundle\Entity\Preset $presets
     * @return Schedule
     */
    public function addPreset(\FS\CamBundle\Entity\Preset $preset)
    {
        $this->presets[] = $preset;

        return $this;
    }

    /**
     * Remove presets
     *
     * @param \FS\CamBundle\Entity\Preset $presets
     */
    public function removePreset(\FS\CamBundle\Entity\Preset $preset)
    {
        $this->presets->removeElement($preset);
    }

    /**
     * Get presets
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getPresets()
    {
        return $this->presets;
    }

    public function removePresets()
    {
        $this->presets = new ArrayCollection();

        return $this;
    }

    public function timeInRange(\DateTime $now)
    {
        $nowTime = $now->format('Gis');
        $startTime = $this->getStartAt()->format('Gis');
        $endTime = $this->getStopAtTime()->format('Gis');

        return ($nowTime >= $startTime) && ($nowTime <= $endTime);
    }

    public function getStopAtTime()
    {
        $startAt = clone $this->getStartAt();

        return $startAt->modify('+ ' . $this->getDuration() . ' seconds');
    }

    public function getRecording()
    {
        return $this->recording;
    }

    public function setRecording($recording = null)
    {
        $this->recording = $recording;

        return $this;
    }

    public function skipNextStart()
    {
        $this->setDoNotStartUntil($this->getNextStopAt());
        //}
    }

    /**
     * @return User
     */
    public function getCreatedBy()
    {
        return $this->createdBy;
    }

    /**
     * @param User $createdBy
     */
    public function setCreatedBy($createdBy)
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    /**
     * Set retentionRule
     *
     * @param RetentionRule $retentionRule
     * @return Schedule
     */
    public function setRetentionRule($retentionRule)
    {
        $this->retentionRule = $retentionRule;

        return $this;
    }

    /**
     * Get retentionRule
     *
     * @return RetentionRule
     */
    public function getRetentionRule()
    {
        return $this->retentionRule;
    }

    public function getObjectName()
    {
        return $this->name;
    }


    public static function sortSchedulesByStartDate(array $schedules, $type)
    {
        if ($type == 'ASC') {
            usort($schedules, ['FS\CamBundle\Entity\Schedule', 'compareByStartDateBig']);
        } else {
            usort($schedules, ['FS\CamBundle\Entity\Schedule', 'compareByStartDateLow']);
        }

        return $schedules;
    }

    public static function sortSchedulesByEndDate(array $schedules, $type)
    {
        if ($type == 'ASC') {
            usort($schedules, ['FS\CamBundle\Entity\Schedule', 'compareByEndDateBig']);
        } else {
            usort($schedules, ['FS\CamBundle\Entity\Schedule', 'compareByEndDateLow']);
        }

        return $schedules;
    }

    public function isBlocked(array $schedules)
    {
        /** @var Schedule $schedule */
        foreach($schedules as $schedule)
        {
            if (
                ($this->getId() == $schedule->getId())
                ||
                ($this->getRoom() != $schedule->getRoom())
            ) {
                continue;
            }
            if (
                (
                    $this->getNextStartAt() <= $schedule->getNextStartAt()
                    &&
                    $this->getNextStartAt() <= $schedule->getNextStopAt()
                )
                &&
                (
                    $this->getNextStopAt() >= $schedule->getNextStartAt()
                    &&
                    $this->getNextStopAt() >= $schedule->getNextStopAt()

                )
            ) {
                return $schedule;
            }
        }
        return false;
    }

    public static function compareByStartDateBig(Schedule $s1, Schedule $s2)
    {
        if ($s1->getNextStartAt() == $s2->getNextStartAt()) {
            return 0;
        }
        return ($s1->getNextStartAt() > $s2->getNextStartAt()) ? 1 : -1;
    }

    public static function compareByStartDateLow(Schedule $s1, Schedule $s2)
    {
        if ($s1->getNextStartAt() == $s2->getNextStartAt()) {
            return 0;
        }
        return ($s1->getNextStartAt() > $s2->getNextStartAt()) ? -1 : 1;
    }

    public static function compareByEndDateBig(Schedule $s1, Schedule $s2)
    {
        if ($s1->getNextStopAt() == $s2->getNextStopAt()) {
            return 0;
        }
        return ($s1->getNextStopAt()) > $s2->getNextStopAt() ? 1 : -1;
    }

    public static function compareByEndDateLow(Schedule $s1, Schedule $s2)
    {
        if ($s1->getNextStopAt() == $s2->getNextStopAt()) {
            return 0;
        }
        return ($s1->getNextStopAt() > $s2->getNextStopAt()) ? -1 : 1;
    }
}
