<?php

namespace FS\CamBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use FS\BasicBundle\Model\Loggable;
use FS\CamBundle\Template\TemplateTypes;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Entity\Container;
use FS\UserBundle\Model\ContainerizedInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Doctrine\Common\Collections\ArrayCollection;


/**
 * Template
 *
 * @ORM\Table(name="templates")
 * @ORM\Entity(repositoryClass="FS\CamBundle\Repository\TemplateRepository")
 * @UniqueEntity("name")
 */
class Template implements Loggable, ContainerizedInterface
{
    use FSContainerTrait;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=128)
     * @Assert\NotBlank
     */
    private $type;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="FS\CamBundle\Entity\TemplateField", mappedBy="template", cascade={"all"})
     */
    private $fields;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean")
     */
    private $hidden;

    public function __construct()
    {
        $this->fields = new ArrayCollection();
        $this->type = TemplateTypes::TYPE_INFO;
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param  string $name
     * @return Exception
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return ArrayCollection | TemplateField[]
     */
    public function getFields()
    {
        return $this->fields;
    }

    public function addField(TemplateField $field)
    {
        $this->fields[] = $field;
        $field->setTemplate($this);

        return $this;
    }

    public function removeField(TemplateField $field)
    {
        $this->fields->removeElement($field);
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     *
     * @return Template
     */
    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    public function getTemplateFullTypeName()
    {
        if ($this->type == TemplateTypes::TYPE_INFO) {
            return "Information";
        } elseif ($this->type == TemplateTypes::TYPE_MARKER) {
            return "Marker";
        } else {
            return "Unknown Type";
        }
    }

    public function __toString()
    {
        return $this->name;
    }

    /**
     * Set hidden
     *
     * @param boolean $hidden
     * @return Template
     */
    public function setHidden($hidden)
    {
        $this->hidden = $hidden;

        return $this;
    }

    /**
     * Get hidden
     *
     * @return boolean
     */
    public function getHidden()
    {
        return $this->hidden;
    }

    public function getObjectName()
    {
        return $this->name;
    }

    /**
     * Get default data for template
     *
     * @return array
     */
    public function getTemplateDefaultData()
    {
        $data = [];
        $fields = $this->getFields();

        foreach ($fields as $field) {
            if ($field->getEnabled()) {
                switch ($field->getType()) {
                    case TemplateField::TYPE_TEXT :
                        $data[$field->getId()] = $field->getDefaultData();
                        break;
                    case TemplateField::TYPE_DROPDOWN :
                        $data[$field->getId()] = explode(',',$field->getDefaultData())[0];
                        break;
                }
            }
        }
        return $data;
    }
}
