<?php

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\Filter;
use FS\CamBundle\Entity\FilterTemplate;
use FS\CamBundle\Entity\Room;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroup;
use Lexik\Bundle\FormFilterBundle\Filter\FilterOperands;
use Lexik\Bundle\FormFilterBundle\Filter\Query\QueryInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Security\Core\SecurityContext;

class RecordFilterType extends AbstractType
{
    /**
     * @var ObjectManager
     */
    private $om;

    /**
     * @var SecurityContext
     */
    private $securityContext;

    /**
     * @param ObjectManager $om
     * @param SecurityContext $securityContext
     */
    public function __construct(ObjectManager $om, SecurityContext $securityContext)
    {
        $this->om = $om;
        $this->securityContext = $securityContext;
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {

        /** @var User $user */
        $user = $this->securityContext->getToken()->getUser();

        $builder
            ->add(
                'name',
                'filter_text',
                [
                    'condition_pattern' => FilterOperands::STRING_BOTH,
                    'attr' => ['placeholder' => 'search criteria'],
                    'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                        if (!empty($values['value'])) {
                            /** @var \Doctrine\ORM\QueryBuilder $qb */
                            $qb = $filterQuery->getQueryBuilder();

                            $value = $filterQuery->getExpr()->literal('%' . $values['value'] . '%');
                            $qb->andWhere(
                                $filterQuery->getExpr()->orx(
                                    $filterQuery->getExpr()->like('record.name', $value),
                                    $filterQuery->getExpr()->like('record.templateData', $value),
                                    $filterQuery->getExpr()->like('room.name', $value),
                                    $filterQuery->getExpr()->like('camera.name', $value),
                                    $filterQuery->getExpr()->like('creator.username', $value),
                                    $filterQuery->getExpr()->like('creator.displayName', $value),
                                    $filterQuery->getExpr()->like('markers.name', $value),
                                    $filterQuery->getExpr()->like('markers.templateData', $value)
                                )
                            );
                        }
                    },
                ]
            )
            ->add(
                'startedAt',
                'filter_date_range',
                [
                    'left_date_options' => ['widget' => 'single_text', 'format' => 'MM/dd/yyyy'],
                    'right_date_options' => ['widget' => 'single_text', 'format' => 'MM/dd/yyyy'],
                    'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                        if (!empty($values['value'])) {
                            /** @var \Doctrine\ORM\QueryBuilder $qb */
                            $qb = $filterQuery->getQueryBuilder();

                            $from = $values['value']['left_date'][0];
                            $to = $values['value']['right_date'][0];

                            if (!empty($from)) {
                                $qb->andWhere('record.startedAt > :from')
                                    ->setParameter('from', $from->format('Y-m-d'));
                            }
                            if (!empty($to)) {
                                $qb->andWhere('record.startedAt < :to')
                                    ->setParameter('to', $to->modify('+ 1 day')->format('Y-m-d'));
                            }

                        }
                    },
                ]
            )
            ->add('userFilter', 'filter_entity',
                [
                    'class' => 'FSCamBundle:Filter',
                    'property' => 'id',
                    'apply_filter' => function (QueryInterface $filterQuery, $field, $values) {
                        $qb = $filterQuery->getQueryBuilder();
                        if (!empty($values['value'])) {
                            /** @var Filter $filter */
                            $filter = $values['value'];
                            $rooms = $filter->getFilterRooms();
                            if ($rooms->count() > 0) {
                                $roomsId = [];
                                /** @var Room $room */
                                foreach ($rooms as $room) {
                                    $roomsId[] = $room->getId();
                                }
                                $qb->andWhere(
                                    $filterQuery->getExpr()->in('room.id', ':fRooms')
                                )
                                    ->setParameter(':fRooms', $roomsId);
                            }
                            $userGroups = $filter->getFilteredUserGroups();
                            if ($userGroups->count() > 0) {
                                $userGroupsId = [];
                                /** @var UserGroup $userGroup */
                                foreach ($userGroups as $userGroup) {
                                    $userGroupsId[] = $userGroup->getId();
                                }
                                $qb->andWhere(
                                    $filterQuery->getExpr()->in('creator.userGroup', ':fUserGroup')
                                )
                                    ->setParameter(':fUserGroup', $userGroupsId);
                            }

                            $users = $filter->getFilteredUsers();

                            if ($users->count() > 0) {
                                $usersId = [];
                                /** @var User $userGroup */
                                foreach ($users as $user) {
                                    $usersId[] = $user->getId();
                                }
                                $qb->andWhere(
                                    $filterQuery->getExpr()->in('creator.id', ':fUsers')
                                )
                                    ->setParameter(':fUsers', $usersId);
                            }

                            $info = $filter->getFilterInfoData();

                            if ($info->count() > 0) {
                                /** @var FilterTemplate $filterTemplate */
                                foreach ($info as $filterTemplate) {
                                    if ($filterTemplate->getEnabled()) {
                                        $value = $filterQuery->getExpr()->literal('%' . $filterTemplate->getFilterText() . '%');
                                        switch ($filterTemplate->getFilterType()) {

                                            case FilterTemplate::FILTER_TEMPLATE_TYPE_RECORD:
                                                $qb->andWhere(
                                                    $filterQuery->getExpr()->like('record.name', $value)
                                                );
                                                break;
                                            case FilterTemplate::FILTER_TEMPLATE_TYPE_MARKER:
                                                $qb->andWhere(
                                                    $filterQuery->getExpr()->like('markers.templateData', $value)
                                                );
                                                break;

                                            case FilterTemplate::FILTER_TEMPLATE_TYPE_EVENT:
                                                $qb->andWhere(
                                                    $filterQuery->getExpr()->like('markers.name', $value)
                                                );
                                                break;
                                            case FilterTemplate::FILTER_TEMPLATE_TYPE_INFORMATION:
                                            case FilterTemplate::FILTER_TEMPLATE_TYPE_REGULAR:
                                            default:
                                                $qb->andWhere(
                                                    $filterQuery->getExpr()->orx(
                                                        $filterQuery->getExpr()->like('record.templateData', $value),
                                                        $filterQuery->getExpr()->like('markers.templateData', $value)
                                                    )
                                                );
                                                break;
                                        }
                                    }

                                }
                            }
                            $from = new \DateTime("now");
                            $to = new \DateTime("now");
                            switch ($filter->getFilterDateType()) {
                                case Filter::FILTER_DATE_TYPE_NONE:
                                    /*NOP*/
                                    break;
                                case Filter::FILTER_DATE_TYPE_DAYS:
                                    /**
                                     * Filter working process for DAYS
                                     * if 0 = today it between day start and day end
                                     *
                                     * if -{countDays} in between -some days to eternity!!!(end of the day)
                                     *
                                     * /// FOR SCHEDULE!
                                     * if +{countDays} in between now + some days(???) how does it will be work i doesn`t know
                                     */
                                    if ($filter->getDays() == 0 || $filter->getDays() > 0) {
                                        // For find in this day we must start query from day start to day end
                                        $qb->andWhere(
                                            $filterQuery->getExpr()->between('record.startedAt', ':startD', ":endD")
                                        );

                                        $from->modify('today');
                                        $to->modify('tomorrow')->modify('-1 second');

                                        $qb->setParameter('startD', $from->format('Y-m-d H:i:s'));
                                        $qb->setParameter('endD', $to->format('Y-m-d H:i:s'));
                                    } else if ($filter->getDays() < 0) {
                                        $from->modify('today'); // set for day Start
                                        $from->modify($filter->getDays() . "day");
                                        $qb->andWhere('record.startedAt >= :fromThis');
                                        $qb->setParameter('fromThis', $from->format('Y-m-d H:i:s'));
                                    }
                                    break;
                                case Filter::FILTER_DATE_TYPE_WEEKS:
                                    /**
                                     * Week Filter
                                     * if this week we find date from start this week to end this week;
                                     *
                                     * if -{weeks} from -weeks to ETERNITY!!!!
                                     * ///FOR SCHEDULE!
                                     * if +{weeks} i
                                     */
                                    if ($filter->getWeeks() == 0){
                                        $weekStart = new \DateTime ("monday this week");
                                        $weekEnd = new \DateTime("next monday");
                                        $qb->andWhere('record.startedAt >= :fromThis');
                                        $qb->andWhere('record.startedAt <= :toThis');
                                        $qb->setParameter('fromThis', $weekStart->format('Y-m-d H:i:s'));
                                        $qb->setParameter('toThis', $weekEnd->modify('-1 second')->format('Y-m-d H:i:s'));
                                    }
                                    else if ($filter->getWeeks() > 0) {
                                        $weekStart = new \DateTime ("monday this week");
                                        $qb->andWhere('record.startedAt <= :toThis');
                                        $qb->setParameter('toThis', $weekStart->format('Y-m-d H:i:s'));

                                    } else if ($filter->getWeeks() < 0) {
                                        $qb->andWhere('record.startedAt >= :fromThis');
                                        $weekStart = new \DateTime ("monday this week");
                                        $weekStart->modify($filter->getWeeks() . "week");
                                        $qb->setParameter('fromThis', $weekStart->format('Y-m-d H:i:s'));
                                    }
                                    break;
                                case Filter::FILTER_DATE_TYPE_RANGE:
                                    // Only 1 query from day start to day end;
                                    $qb->andWhere(
                                        $filterQuery->getExpr()->between('record.startedAt', ':startD', ":endD")
                                    );
                                    $qb->setParameter('startD',
                                        $filter->getRangeStart()
                                            ->format('Y-m-d H:i:s')
                                    );

                                    $qb->setParameter('endD',
                                        $filter->getRangeEnd()
                                            ->modify('tomorrow')
                                            ->modify('-1 second')
                                            ->format('Y-m-d H:i:s')
                                    );

                                    break;
                                default:
                                    break;
                            }
                        }
                    }

                ]);

    }

    /*
     * http://ds.loc/record/review/?record_filter%5BstartedAt%5D%5Bleft_date%5D=&record_filter%5BstartedAt%5D%5Bright_date%5D=&record_filter%5Bname%5D=&record_filter%5BuserFilter%5D=29&submit-filter=
     */

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'csrf_protection' => false,
                'validation_groups' => array('filtering') // avoid NotBlank() constraint-related message
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'record_filter';
    }
}
