<?php

namespace FS\CamBundle\Form\Type;

use Doctrine\Common\Persistence\ObjectManager;
use FS\CamBundle\Entity\RetentionRule;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Schedule;
use FS\CamBundle\Entity\Template;
use FS\UserBundle\Entity\User;
use FS\UserBundle\Entity\UserGroupRight;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Security\Core\SecurityContext;

class ScheduleType extends AbstractType
{
    /** @var SecurityContext */
    private $security;

    /**
     * {@inheritdoc}
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $infoTab = $builder->create(
            'info',
            'tab',
            array(
                'label' => 'Information',
                'inherit_data' => true,
            )
        );

        $infoTab
            ->add(
                'room',
                'entity',
                [
                    'empty_value' => 'Select the room',
                    'property' => 'name',
                    'class' => 'FS\CamBundle\Entity\Room',
                    'choices' => $this->getUser()->getUserGroup()->getRooms()
                ]
            )
            ->add('name', null, ['label' => 'Recording name'])
            ->add(
                'templateData',
                'template_data',
                [
                    'template_aware' => $builder->getData(),
                ]
            );

        $scheduleTab = $builder->create(
            'schedule',
            'tab',
            array(
                'label' => 'Schedule',
                'inherit_data' => true,
            )
        );
        $scheduleTab
            ->add(
                'startAt',
                'fs_datetime',
                ['label' => false]
            );

        $scheduleTab
            ->add(
                'duration',
                'fs_duration'
            );
        $schedule = $builder->getData();

        $hasRecur = null !== $schedule->getRecur();
        $scheduleTab->add(
            'hasRecur',
            'checkbox',
            [
                'label' => 'Recurrence',
                'widget_checkbox_label' => 'widget',
                'data' => $hasRecur,
                'mapped' => false
            ]
        );
        $scheduleTab->add('recur', new RecurType(), ['label' => false]);

        $builder
            ->add($infoTab)
            ->add($scheduleTab);

        $user = $this->getUser();
        if ($user->hasRight(UserGroupRight::GRANT_SCHEDULE__SHARING)) {
            $shareTab = $builder->create(
                'sharing',
                'tab',
                array(
                    'label' => 'Sharing',
                    'inherit_data' => true,
                )
            );
            $shareTab->add('accessedByUsersAndGroups', 'users_and_groups', ['owner' => $builder->getData()->getCreatedBy()]);
            $builder->add($shareTab);

        }
        if ($user->hasRight(UserGroupRight::GRANT_SCHEDULE__RETENTION)) {
            $retentionTab = $builder->create(
                'retention',
                'tab',
                array(
                    'label' => 'Retention',
                    'inherit_data' => true,
                )
            );
            $retentionTab->add('retentionRule', 'retention', ['label' => false, 'cascade_validation' => true,]);
            $builder->add($retentionTab);
        }
        if ($user->hasRight(UserGroupRight::GRANT_SCHEDULE__CONTROL)) {
            $controlTab = $builder->create(
                'control',
                'tab',
                array(
                    'label' => 'Control',
                    'inherit_data' => true,
                )
            );
            $builder->add($controlTab);
        }

        $formModifier = function (FormInterface $form, Room $room = null) {
            if ($form->has('control')) {
                $controlTab = $form->get('control');
                $controlTab->add(
                    'cameras',
                    'form',
                    array(
                        'label' => 'Go to Preset on Start',
                        'label_attr' => ['class' => 'font-bold display-block'],
                        'inherit_data' => true,
                    )
                );
                $cameras = $controlTab->get("cameras");
                if (null !== $room) {
                    foreach ($room->getCameras() as $id => $camera) {
                        $cameras->add(
                            $id,
                            'entity',
                            array(
                                'label' => $camera->getName(),
                                'label_attr' => ['class'=>'preset-select'],
                                'class' => 'FSCamBundle:Preset',
                                'empty_value' => 'Select the preset',
                                'choices' => $camera->getPresets(),
                                'property' => 'name',
                                'mapped' => false,
                                'attr' => ['class'=>'preset-value']

                            )
                        );
                    }
                }
            }
        };

        $builder->addEventListener(
            FormEvents::PRE_SET_DATA,
            function (FormEvent $event) use ($formModifier) {
                $data = $event->getData();

                $formModifier($event->getForm(), null === $data ? null : $data->getRoom());
            }
        );

        $builder->get('info')->get('room')->addEventListener(
            FormEvents::POST_SUBMIT,
            function (FormEvent $event) use ($formModifier) {
                // It's important here to fetch $event->getForm()->getData(), as
                // $event->getData() will get you the client data (that is, the ID)
                $room = $event->getForm()->getData();
                // since we've added the listener to the child, we'll have to pass on
                // the parent to the callback functions!
                $formModifier($event->getForm()->getParent()->getParent(), $room);
            }

        );


        $builder->addEventListener(
            FormEvents::SUBMIT,
            function (FormEvent $event) {
                $hasRecur = $event->getForm()->get('schedule')->get('hasRecur')->getData();

                if (!$hasRecur) {
                    /** @var \FS\CamBundle\Entity\Schedule $schedule */
                    $schedule = $event->getForm()->getData();
                    $schedule->setRecur();
                    $event->setData($schedule);
                }
            }
        );

    }

    /**
     * {@inheritdoc}
     */
    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(
            array(
                'data_class' => 'FS\CamBundle\Entity\Schedule',
                'cascade_validation' => true,
            )
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'schedule';
    }

    /**
     * @param SecurityContext $security
     */
    public function setSecurity($security)
    {
        $this->security = $security;
    }

    /**
     * @return User
     */
    private function getUser()
    {
        return $this->security->getToken()->getUser();
    }

}
