<?php

namespace FS\CamBundle\Utils;

use Buzz\Browser;
use Buzz\Exception\ClientException;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Wowza;
use FS\UserBundle\Entity\User;
use Predis\Client as RedisClient;
use Symfony\Component\Security\Core\SecurityContext;


class Recorder
{
    const REDIS_CHANNEL = 'recording:status';

    private $buzz;
    private $client;
    private $redis;
    private $securityContext;
    private $wowzaManager;

    public function __construct(
        WowzaSenderClient $client,
        Browser $buzz,
        RedisClient $redis,
        SecurityContext $securityContext,
        WowzaManager $wowzaManager
    )
    {
        $this->buzz = $buzz;
        $this->client = $client;
        $this->redis = $redis;
        $this->securityContext = $securityContext;
        $this->wowzaManager = $wowzaManager;
    }

    /**
     * @param array $requests
     * @return string
     * @internal param array $request
     */
    public function send(array $requests)
    {
        $content = [];
        foreach ($requests as $request) {
            /** @var Wowza $wowza */
            $wowza = $request['wowza'];
            $contentRaw = $this->client
                ->post(
                    'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                    ['Content-Type' => 'application/json'],
                    json_encode($request['request'])
                );
            $content[] = ($contentRaw['content'] == "false\n") ? false : true;
        }
        $contentLen = count($content);
        foreach ($content as $res) {
            if ($res == false) {
                $contentLen -= 1;
            }
        }

        return ($contentLen == 0) ? false : true;
    }

    private function getRoomCameras(Room $room, Wowza $wowza, array $presets = [])
    {

        $cameras = [];
        /** @var \FS\CamBundle\Entity\Camera $camera */
        $i = 0;
        foreach ($room->getCameras() as $id => $camera) {
            if($camera->getDeviceType() == Camera::DEVICE_TYPE_CARD_READER){
                continue;
            }
            if ($wowza->getCameras()->contains($camera)) {
                $cameras[$i] = [
                    'id' => (string)$camera->getId(),
                    'weight' => (string)$id
                ];
                if (!empty($presets)) {
                    foreach ($presets as $preset) {
                        if ($preset->getCamera()->getId() == $camera->getId()) {
                            $cameras[$i]['preset'] = (string)$preset->getData();
                        }
                    }

                }
                ++$i;
            }
        }
        return $cameras;
    }

    public function startRecording(Record $record, array $presets = [], $controllerIp = null, $readerNumber = null)
    {
        if (!$record->getId()) {
            new \Exception('Record id isn\'t defined');
        }
        // '{"type": "scheduled", "recordId": "23456", "command": "start",
        // "cameras": [{"id": "1", "weight": "1"}, {"id": "2", "weight": "2"}]}'

        $room = $record->getRoom();
        $wowzas = $room->getWowzas();
        $result = [];
        //get record maximum duration
        $recordDuration = 0;
        $owner = $record->getCreatedBy();
        if ($owner != null && is_object($owner)) {
            $userGroup = $owner->getUserGroup();
            if ($userGroup != null && is_object($userGroup)) {
                $groupRecordDuration = $userGroup->getRecordDuration();
                if ($groupRecordDuration != null || $groupRecordDuration != 0) {
                    $recordDuration = $groupRecordDuration;
                }
            }
        }

        $i = 0;
        /** @var Wowza $wowza */
        foreach ($wowzas as $wowza) {
            $result[$i]['wowza'] = $wowza;

            $result[$i++]['request'] = [
                'type' => 'scheduled',
                'recordId' => (string)$record->getId(),
                'command' => 'start',
                'duration' => (string)$recordDuration,
                'cameras' => $this->getRoomCameras($room, $wowza, $presets),
                'controllerIp' => (string)($controllerIp != null)?$controllerIp:'',
                'readerNumber' => (string)($readerNumber != null)?$readerNumber:'',
            ];
        }

        $result = $this->send(
            $result
        );
        //if record doesnt start return false;

        if (!$result) {
            return $result;
        }
        //TODO:DEBUUUG!!!
        /*if ($result) {
            return !$result;
        }*/

        if ($record->getRoom()->getIsCameraIO() && $record->getRoom()->getCameraIO() != null) {
            $camera = $record->getRoom()->getCameraIO();

            $host = 'http://' . $camera->getUsername() . ':' . $camera->getPassword() . '@' . $camera->getIp() . ':' . $camera->getPort();

            $response = new \Buzz\Message\Response();
            $request = new \Buzz\Message\Request('GET', '/', $host);

            $client = $this->buzz->getClient();

            $resource = '/axis-cgi/io/port.cgi?action=2:/';

            $request->setResource($resource);
            try {
                $client->send($request, $response);
            } catch (ClientException $e) {
                /*NOP*/
            }

        }

        /** @var User $user */
        $user = $this->getUser();
        $accessToRecord = 0;

        if ($user != null && is_object($user)) {
            $accessToRecord = ($user->hasAccessToRecord($record) == true) ? 1 : 0;
        }
        $this->redis->publish(
            self::REDIS_CHANNEL,
            json_encode(
                [
                    'action' => 'start',
                    'room_id' => $record->getRoom()->getId(),
                    'record_id' => $record->getId(),
                    'recordAuthor' => $record->getCreatedBy()->getId(),
                    'accessToView' => $accessToRecord,
                ])
        );

        return $result;
    }

    public function stopRecording(Record $record)
    {
        //'{"type": "scheduled", "recordId": "23456", "command": "stop"}'

        $room = $record->getRoom();
        $wowzas = $room->getWowzas();
        $result = [];

        $i = 0;
        /** @var Wowza $wowza */
        foreach ($wowzas as $wowza) {
            $result[$i]['wowza'] = $wowza;

            $result[$i++]['request'] = [
                'type' => 'scheduled',
                'recordId' => (string)$record->getId(),
                'command' => 'stop'
            ];
        }

        $result = $this->send(
            $result
        );

        if ($record->getRoom()->getIsCameraIO() && $record->getRoom()->getCameraIO() != null) {
            $camera = $record->getRoom()->getCameraIO();

            $host = 'http://' . $camera->getUsername() . ':' . $camera->getPassword() . '@' . $camera->getIp() . ':' . $camera->getPort();

            $response = new \Buzz\Message\Response();
            $request = new \Buzz\Message\Request('GET', '/', $host);

            $client = $this->buzz->getClient();

            $resource = '/axis-cgi/io/port.cgi?action=2:\\';

            $request->setResource($resource);
            try {
                $client->send($request, $response);
            } catch (ClientException $e) {
                /*NOP*/
            }

        }
        $this->redis->publish(
            self::REDIS_CHANNEL,
            json_encode(['action' => 'stop', 'room_id' => $record->getRoom()->getId()])
        );

        $now = new \DateTime();

        $record->setDuration($now->getTimestamp() - $record->getStartedAt()->getTimestamp());
        return $result;
    }


    public function getTime(Record $record)
    {
        //'{"command": "getTime", "recordId": "23456"}'
        $wowza = $record->getRoom()->getWowzas()[0];
        $response = json_decode($this->client
            ->post(
                'http://' . $wowza->getAddress() . ':8086/dustinRecorder',
                ['Content-Type' => 'application/json'],
                json_encode([
                    'recordId' => (string)$record->getId(),
                    'command' => 'getTime'
                ])
            )['content']);


        return $response !== null ? $response->time : null;
    }

    /**
     * Method for cut record
     * Send time and records
     *
     * @param Record $recordToClip
     * @param Record $clipRecord
     * @param $startTime
     * @param $stopTime
     *
     * @return array|string
     */
    public function clipRecord(Record $recordToClip, Record $clipRecord, $startTime, $stopTime)
    {
        if (!$recordToClip->getId() || $clipRecord->getId()) {
            new \Exception('Record id isn\'t defined');
        }

        $room = $recordToClip->getRoom();
        $wowzas = [];
        if($recordToClip->getRecordType() == Record::RECORD_TYPE_UPLOAD){
            $wowzas[]= $this->wowzaManager->getLocalWowza();
        }else {
            if($room != null){
                $wowzas = $room->getWowzas();
            } else{
                $wowzas = $this->wowzaManager->getAllWowzas();
            }

        }

        $result = [];

        $i = 0;
        /** @var Wowza $wowza */
        foreach ($wowzas as $wowza) {
            $result[$i]['wowza'] = $wowza;

            $result[$i++]['request'] = [
                'command' => 'cut',
                'recordId' => (string)$recordToClip->getId(),
                'clipRecordId' => (string)$clipRecord->getId(),
                'start' => (string)$startTime,
                'stop' => (string)$stopTime,
                'type' => 'live',
                'userId' => (string)$recordToClip->getId()
            ];
        }

        $result = $this->send(
            $result
        );

        return $result;
    }

    protected function getUser()
    {
        if (null === $token = $this->securityContext->getToken()) {
            return null;
        }

        if (!is_object($user = $token->getUser())) {
            return null;
        }

        return $user;
    }
}
