<?php

namespace FS\UserBundle\Controller;

use FS\UserBundle\Entity\User;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Method;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;


/**
 * Class UserAdminController
 * @Route("/admin/user")
 * @Security(expression="user.hasRight('admin--users-and-groups') || is_granted('ROLE_ADMIN')")
 */
class UserAdminController extends Controller
{
    /**
     * @Route(path="/", name="admin_user")
     * @Template()
     */
    public function indexAction()
    {
        return [];
    }


    /**
     * Displays a form to create a new User entity.
     *
     * @Route("/new", name="admin_user_new")
     * @Template()
     */
    public function newAction()
    {
        $user = new User();
        $form = $this->createForm('user', $user);

        return array(
          'user' => $user,
          'form' => $form->createView(),
        );
    }

    /**
     * Creates a new User entity.
     *
     * @Route("/create", name="admin_user_create")
     * @Method("POST")
     * @Template("FSUserBundle:UserAdmin:new.html.twig")
     */
    public function createAction(Request $request)
    {
        $user = new User();
        $form = $this->createForm('user', $user);

        if ($form->submit($request)->isValid()) {
            $user->addSelfToAccessedUsers();
            $user->setEnabled(true);
            $user->setType(User::TYPE_LOCAL);

            $this->container->get('fos_user.user_manager')->updateUser($user);

            $this->get('session')->getFlashBag()->add(
                'note',
                'New user has been successfully added'
            );

            return $this->redirect($this->generateUrl('admin_user_edit', ['id' => $user->getId()]));
        }

        return array(
          'user' => $user,
          'form' => $form->createView(),
        );
    }

    /**
     * Displays a form to edit an existing User entity.
     *
     * @Route("/edit/{id}", name="admin_user_edit")
     * @Template()
     */
    public function editAction(User $user)
    {
        $this->checkUserStatus($user);

        $editForm = $this->createForm('user', $user);

        return array(
          'user' => $user,
          'form' => $editForm->createView(),
        );
    }

    /**
     * Edits an existing User entity.
     *
     * @Route("/update/{id}", name="admin_user_update")
     * @Method("POST")
     * @Template("FSUserBundle:UserAdmin:edit.html.twig")
     */
    public function updateAction(User $user, Request $request)
    {
        $this->checkUserStatus($user);

        $editForm = $this->createForm('user', $user);
        if ($editForm->submit($request)->isValid()) {
            $this->container->get('fos_user.user_manager')->updateUser($user);
        }

        return $this->redirect($this->generateUrl('admin_user_edit', array(
          'id' => $user->getId())
        ));
    }

    /**
     * Displays a form to delete a User entity.
     *
     * @Route("/{id}/remove", name="admin_user_remove")
     * @Template()
     */
    public function removeAction(User $user, Request $request)
    {
        $this->checkUserStatus($user);

        $form = $this->createDeleteForm($user->getId());

        return array('user' => $user, 'form' => $form->createView());
    }

    /**
     * Deletes a User entity.
     *
     * @Route("/{id}/delete", name="admin_user_delete")
     * @Method("POST")
     */
    public function deleteAction(User $user, Request $request)
    {
        $this->checkUserStatus($user);

        $form = $this->createDeleteForm($user->getId());
        if ($form->bind($request)->isValid()) {
            $em = $this->getDoctrine()->getManager();

            $user->setUsernameCanonical(hash('md5', $user->getUsername().$user->getId()));
            $user->setEmailCanonical(hash('md5', $user->getEmail().$user->getId()));
            $user->setDeletedAt(new \DateTime());

            $em->persist($user);
            $em->flush();

            $this->get('session')->getFlashBag()->add(
                'note',
                'User has been successfully deleted'
            );
        }

        return $this->redirect($this->generateUrl('admin_user'));
    }


    /**
     * Create Delete form
     *
     * @param  integer $id
     * @return FormBuilder
     */
    protected function createDeleteForm($id)
    {
        return $this->createFormBuilder(array('id' => $id))
          ->add('id', 'hidden')
          ->getForm();
    }

    /**
     * @param User $user
     */
    protected function checkUserStatus(User $user)
    {
        if ($user->isDeleted()) {
            throw new NotFoundHttpException(sprintf('%s object not found',get_class($user)));
        }
    }


} 