<?php

namespace FS\UserBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use FS\UserBundle\Model\ContainerizedInterface;
use FS\UserBundle\Twig\Extension\TruncateStringTrait;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity(repositoryClass="FS\UserBundle\Repository\ContainerRepository")
 * @ORM\Table(name="container")
 */
class Container
{
    use TruncateStringTrait;

    /**
     * @var integer
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @ORM\Column(type="string",length=255)
     *
     * @Assert\NotBlank
     */
    private $name;

    /**
     * @ORM\OneToMany(targetEntity="FS\UserBundle\Entity\Container", mappedBy="parent", cascade={"all"})
     */
    private $children;

    /**
     * @Assert\NotBlank
     * @ORM\ManyToOne(targetEntity="Container", inversedBy="children")
     * @ORM\JoinColumn(name="c_id", referencedColumnName="id")
     */
    private $parent;
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->children = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer 
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Container
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string 
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return string
     */
    public function getTruncatedName()
    {
        return $this->truncateString($this->getName(), 25);
    }

    /**
     * Add children
     *
     * @param \FS\UserBundle\Entity\Container $children
     * @return Container
     */
    public function addChild(\FS\UserBundle\Entity\Container $children)
    {
        $this->children[] = $children;

        return $this;
    }

    /**
     * Remove children
     *
     * @param \FS\UserBundle\Entity\Container $children
     */
    public function removeChild(\FS\UserBundle\Entity\Container $children)
    {
        $this->children->removeElement($children);
    }

    /**
     * Get children
     *
     * @return \Doctrine\Common\Collections\Collection 
     */
    public function getChildren()
    {
        return $this->children;
    }

    /**
     * Set parent
     *
     * @param \FS\UserBundle\Entity\Container $parent
     * @return Container
     */
    public function setParent(\FS\UserBundle\Entity\Container $parent = null)
    {
        $this->parent = $parent;

        return $this;
    }

    /**
     * Get parent
     *
     * @return \FS\UserBundle\Entity\Container 
     */
    public function getParent()
    {
        return $this->parent;
    }

    /**
     * @return mixed
     */
    public function __toString()
    {
        return $this->getTruncatedName();
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $result = [];
        $result [] = $this;
        /** @var Container $child */
        foreach($this->getChildren() as $child){
            $result = array_merge($result, $child->toArray());
        }
        return $result;
    }
    /**
     * @param Container $container
     * @return bool
     */
    public function isVisible($container)
    {
        /** @var Container $container */
        /*
           * check if head container
        */
        if($this->getParent() == null){
            return true;
        }
        if($container == null){
            return true;
        }

        /*
         * if parent is container it`s not visible
         */
        if($this->getParent() == $container){
            return false;
        }

        /*
         * if equals return true
         */
        if($this == $container)
            return true;

        if($this->getChildren()->count() == 0){
            return false;
        }
        /*
         * if this container is my child i see it
         */
        if($this->getChildren()->contains($container))
            return true;

        /** @var Container $cContainer */
        foreach($this->getChildren() as $cContainer)
        {
            /*
             * if one of my child see it i see it
             */
            if($cContainer->isVisible($container)){
                return true;
            }
        }

        return false;
    }

    /**
     * @param Container $userContainer
     * @param Container $entityContainer
     * @return bool
     */
    public static function isContainerVisible(Container $userContainer, Container $entityContainer)
    {
        if($entityContainer == null){
            return true;
        }
        if($userContainer == null){
            return false;
        }

        return $userContainer->isVisible($entityContainer);
    }

    /**
     * @param ContainerizedInterface $entity
     * @param ContainerizedInterface $target
     * @return bool
     */
    public static function isEntityVisible(ContainerizedInterface $entity, ContainerizedInterface $target)
    {
        return self::isContainerVisible($entity->getContainer(), $target->getContainer());
    }

}
