<?php

namespace FS\UserBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\PersistentCollection;
use FOS\UserBundle\Model\User as BaseUser;
use Doctrine\ORM\Mapping as ORM;
use FS\ApiBundle\UserApiTokenizer\FSApiTokenInterface;
use FS\ApiBundle\UserApiTokenizer\FSApiTokenTrait;
use FS\BasicBundle\Model\Loggable;
use FS\CamBundle\Entity\Camera;
use FS\CamBundle\Entity\Filter;
use FS\CamBundle\Entity\Record;
use FS\CamBundle\Entity\RestrictAccess;
use FS\CamBundle\Entity\RestrictAccessWeekDay;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Expiration\ExpirationInterface;
use FS\CamBundle\Expiration\ExpirationTrait;
use FS\CamBundle\RecordAccess\AccessedUsersAndGroupsInterface;
use FS\CamBundle\RecordAccess\AccessedUsersAndGroupsTrait;
use FS\CamBundle\RestrictAccess\RestrictAccessInterface;
use FS\CamBundle\RestrictAccess\RestrictAccessTrait;
use Gedmo\SoftDeleteable\Traits\SoftDeleteableEntity;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Doctrine\Common\Collections\Criteria;
use FS\UserBundle\Validator\Constraints as FSUserAssert;
use Gedmo\Mapping\Annotation as Gedmo;

/**
 * @ORM\Entity(repositoryClass="FS\UserBundle\Repository\UserRepository")
 * @ORM\Table(name="users")
 *
 * @UniqueEntity(
 *     fields={"usernameCanonical"},
 *     errorPath="username",
 *     message="fos_user.username.already_used",
 *     groups={"AdminCreate", "AdminEdit"}
 * )
 */
class User extends BaseUser implements AccessedUsersAndGroupsInterface,
    Loggable,
    RestrictAccessInterface,
    ExpirationInterface,
    FSApiTokenInterface
{
    use AccessedUsersAndGroupsTrait;
    
    use SoftDeleteableEntity;

    use RestrictAccessTrait;
    use ExpirationTrait;

    use FSApiTokenTrait;

    const TYPE_LOCAL = 'local';
    const TYPE_LDAP = 'ldap';
    const TYPE_IO = 'io';
    /**
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @Assert\NotBlank(
     *      groups={"AdminCreate", "AdminEdit"}
     * )
     * @Assert\Length(
     *      min=1,
     *      max=20
     * )
     */
    protected $username;

    /**
     * @Assert\NotBlank(
     *      groups={"Registration", "AdminCreate"}
     * )
     */
    protected $plainPassword;

    /**
     * @ORM\ManyToOne(targetEntity="UserGroup", inversedBy="users")
     * @ORM\JoinColumn(name="group_id", referencedColumnName="id", onDelete="SET NULL")
     **/
    private $userGroup;

    /**
     * @var ArrayCollection | \FS\CamBundle\Entity\Room[]
     * @ORM\ManyToMany(targetEntity="FS\CamBundle\Entity\Room", inversedBy="users")
     * @ORM\JoinTable(name="users_share_rooms",
     *      joinColumns={@ORM\JoinColumn(name="user_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="room_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $sharedRooms;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection | Filter[]
     * @ORM\OneToMany(targetEntity="FS\CamBundle\Entity\Filter", mappedBy="owner", cascade={"remove"})
     */
    private $filters;

    /**
     * @var string
     *
     * @ORM\Column(name="userType", type="string", nullable=true)
     */
    protected $type;

    /**
     * @var string
     *
     * @ORM\Column(name="ldapScript", type="string", nullable=true)
     */
    protected $ldapScript;

    /**
     * @var string
     *
     * @ORM\Column(name="ldapDomain", type="string", nullable=true)
     */
    protected $ldapDomain;

    /**
     * @var string
     *
     * @ORM\Column(name="ldapDN", type="string", nullable=true)
     */
    protected $ldapDN;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true, length=20)
     * @Assert\Length(
     *      min=0,
     *      max=20
     * )
     */
    protected $displayName;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    protected $cardNumber;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     * @FSUserAssert\PIN(
     *      groups={"Registration", "AdminCreate"}
     * )
     */
    protected $pin;

    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", options={"default":false})
     */
    private $disabledAccess = false;

    /**
     * @var LdapSynchronize
     * @ORM\ManyToOne(targetEntity="FS\UserBundle\Entity\LdapSynchronize", inversedBy="usersInSync")
     * @ORM\JoinColumn(name="ldap_sync_id", referencedColumnName="id", onDelete="SET NULL", nullable=true)
     */
    private $ldapSyncSchedule;

    public function __construct()
    {
        parent::__construct();
        // your own logic
        $this->sharedRooms = new ArrayCollection();
        $this->filters = new ArrayCollection();
        $this->disabledAccess = false;
    }

    public function setUsername($username)
    {
        parent::setUsername($username);
        parent::setEmail($username);

        return $this;
    }

    /**
     * @param string $ldapDN
     */
    public function setLdapDN($ldapDN)
    {
        $this->ldapDN = $ldapDN;
    }

    /**
     * @return string
     */
    public function getLdapDN()
    {
        return $this->ldapDN;
    }

    public function getTypes()
    {
        return [self::TYPE_LOCAL, self::TYPE_LDAP, self::TYPE_IO];
    }

    /**
     * @param string $type
     */
    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param mixed $userGroup
     */
    public function setUserGroup($userGroup)
    {
        $this->userGroup = $userGroup;
    }

    /**
     * @return UserGroup
     */
    public function getUserGroup()
    {
        return $this->userGroup;
    }

    /**
     * Add room
     *
     * @param  \FS\CamBundle\Entity\Room $room
     * @return User
     */
    public function addSharedRooms(\FS\CamBundle\Entity\Room $room)
    {
        $this->sharedRooms[] = $room;

        return $this;
    }

    /**
     * Remove room
     *
     * @param \FS\CamBundle\Entity\Room $room
     */
    public function removeSharedRooms(\FS\CamBundle\Entity\Room $room)
    {
        $this->sharedRooms->removeElement($room);
    }

    /**
     * Get presets
     *
     * @return \Doctrine\Common\Collections\Collection | Room[]
     */
    public function getSharedRooms()
    {
        return $this->sharedRooms;
    }

    /**
     * @param string $ldapScript
     */
    public function setLdapScript($ldapScript)
    {
        $this->ldapScript = $ldapScript;
    }

    /**
     * @return string
     */
    public function getLdapScript()
    {
        return $this->ldapScript;
    }

    /**
     * @param string $ldapDomain
     */
    public function setLdapDomain($ldapDomain)
    {
        $this->ldapDomain = $ldapDomain;
    }

    /**
     * @return string
     */
    public function getLdapDomain()
    {
        return $this->ldapDomain;
    }

    public function hasRight($right)
    {
        if (null == $this->getUserGroup()) {
            return false;
        }
        return $this->getUserGroup()->hasRight($right);
    }

    /**
     * @param $parent
     * @return bool
     * @deprecated
     */
    public function hasSubRights($parent){
        /** @var UserGroup $userGroup */
        if ($userGroup = $this->getUserGroup() == null) {
            return false;
        }
        $rights = UserGroupRight::getSubRightsList($parent);
        return true;
    }

    public function hasAccessToRecord(Record $record)
    {
        // return true;

        $user = $this;
        $adminGroup = false;
        if ($user->getUserGroup()) {
            if ($user->getUserGroup()->getId() == 1) {
                $adminGroup = true;
            }
        }
        if (in_array("ROLE_ADMIN", $user->getRoles()) || $adminGroup) {
            return true;
        }

        $userGroup = $this->getUserGroup();
        /* if ($userGroup != null && !$userGroup->isVisible($record)){
             return false;
         }*/
        //$createBy = $record->getCreatedBy();

        $creator = $record->getCreatedBy();
        $creatorGroup = $creator->getUserGroup();

        $userAccessedUsers = $user->getAccessedUsers();
        $userAccessedGroups = $user->getAccessedGroups();

        $userGroupAccessedUsers = $userGroup->getAccessedUsers();
        $userGroupAccessedGroups = $userGroup->getAccessedGroups();

        $groupRooms = $userGroup->getRooms();

        $room = $record->getRoom();

        $sharedRooms = $user->getSharedRooms();

        $userAccessedToRecord = $record->getAccessedByUsers();
        $groupsAccessedToRecord = $record->getAccessedByGroups();

        if (
            ($userGroupAccessedGroups->contains($creatorGroup)
                ||
                $userAccessedGroups->contains($creatorGroup)
                ||
                $userGroupAccessedUsers->contains($creator)
                ||
                $userAccessedUsers->contains($creator)
                ||
                $userAccessedToRecord->contains($user)
                ||
                $groupsAccessedToRecord->contains($user->getUserGroup()))
            ||
            $userGroup->isVisible($record)
            && (
                $userGroupAccessedGroups->contains($creatorGroup)
                ||
                $userAccessedGroups->contains($creatorGroup)
                ||
                $userAccessedToRecord->contains($user)
                ||
                $groupsAccessedToRecord->contains($user->getUserGroup())
            )
        ) {
            return true;
        }

        return false;
    }

    public function __toString()
    {
        return $this->getShowingName();
    }

    public function getObjectName()
    {
        return $this->username;
    }

    /**
     * @return array
     */
    public function getAccessedRoomsToChoice()
    {
        /** @var PersistentCollection $roomsRaw */
        $roomsRaw = $this->getUserGroup()->getRooms();

        $result = [];
        /** @var Room $room */
        foreach ($roomsRaw as $room) {
            $result[$room->getId()] = $room->getName();
        }
        return $result;
    }


    /**
     * Add filters
     *
     * @param \FS\CamBundle\Entity\Filter $filters
     * @return User
     */
    public function addFilter(\FS\CamBundle\Entity\Filter $filters)
    {
        $this->filters[] = $filters;

        return $this;
    }

    /**
     * Remove filters
     *
     * @param \FS\CamBundle\Entity\Filter $filters
     */
    public function removeFilter(\FS\CamBundle\Entity\Filter $filters)
    {
        $this->filters->removeElement($filters);
    }

    /**
     * Get filters
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getFilters()
    {
        return $this->filters;
    }

    public function setDisplayName($displayName)
    {
        $this->displayName = $displayName;
        return $this;
    }

    /**
     * @return string
     */
    public function getDisplayName()
    {
        return $this->displayName;
    }

    /**
     * @return string
     */
    public function getShowingName()
    {
        return $this->displayName ? $this->displayName : $this->username;
    }

    /**
     * @param $cardNumber
     */
    public function setCardNumber($cardNumber)
    {
        $this->cardNumber = $cardNumber;
    }

    /**
     * @return string
     */
    public function getCardNumber()
    {
        return $this->cardNumber;
    }

    public function getUserSimpleData()
    {
        $data = [];
        $data['id'] = $this->getId();
        $data['username'] = $this->getUsername();
        if ($this->getUserGroup() != null) {
            $data['userGroup'] = $this->getUserGroup()->getName();
            $data['userGroupId'] = $this->getUserGroup()->getId();
        } else {
            $data['userGroup'] = "";
            $data['userGroupId'] = "";
        }
        if($this->getPin() != null){
            $data['pin'] = md5($this->getPin());
        }else {
            $data['pin'] = '';
        }
        return $data;
    }

    /**
     * @return string
     */
    public function getPin()
    {
        return $this->pin;
    }

    /**
     * @param string $pin
     */
    public function setPin($pin)
    {
        $this->pin = $pin;
    }

    /**
     * @param $disabledAccess
     * @return $this
     */
    public function setDisabledAccess($disabledAccess)
    {
        $this->disabledAccess = $disabledAccess;
        return $this;
    }

    /**
     * @return bool
     */
    public function getDisabledAccess()
    {
        return $this->disabledAccess;
    }

    /**
     * @return boolean
     */
    public function hasAccessNow()
    {
        if ($this->isAccessToday()) {
            if ($this->getUserGroup() != null) {
                return $this->getUserGroup()->hasAccessNow();
            }
            return true;
        }
        return false;
    }

    public function isAccessExpired()
    {
        if ($this->checkExpiration()) {
            return true;
        }
        if ($this->getUserGroup() != null) {
            return $this->getUserGroup()->isAccessExpired();
        }
        return false;
    }

    public function isDisabledAccess()
    {
        if($this->getDisabledAccess()){
            return true;
        }
        if ($this->getUserGroup() != null) {
            return $this->getUserGroup()->getDisabledAccess();
        }
        return false;
    }

    /**
     * @return LdapSynchronize
     */
    public function getLdapSyncSchedule()
    {
        return $this->ldapSyncSchedule;
    }

    /**
     * @param LdapSynchronize $ldapSyncSchedule
     */
    public function setLdapSyncSchedule($ldapSyncSchedule)
    {
        $this->ldapSyncSchedule = $ldapSyncSchedule;
    }

}
