<?php

namespace FS\UserBundle\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use FS\BasicBundle\Model\Loggable;
use FS\CamBundle\Entity\Expiration;
use FS\CamBundle\Entity\RetentionRule;
use FS\CamBundle\Entity\Room;
use FS\CamBundle\Entity\Template;
use FS\CamBundle\Entity\TemplateField;
use FS\CamBundle\Expiration\ExpirationInterface;
use FS\CamBundle\Expiration\ExpirationTrait;
use FS\CamBundle\RecordAccess\AccessedUsersAndGroupsInterface;
use FS\CamBundle\RecordAccess\AccessedUsersAndGroupsTrait;
use FS\CamBundle\RestrictAccess\RestrictAccessBuilder;
use FS\CamBundle\RestrictAccess\RestrictAccessInterface;
use FS\CamBundle\RestrictAccess\RestrictAccessTrait;
use FS\UserBundle\Container\FSContainerTrait;
use FS\UserBundle\Model\ContainerizedInterface;
use FS\UserBundle\Model\DeepCopyInterface;
use FS\UserBundle\Twig\Extension\TruncateStringTrait;
use Symfony\Component\Validator\Constraints as Assert;


/**
 * @ORM\Entity(repositoryClass="FS\UserBundle\Repository\UserGroupRepository")
 * @ORM\Table(name="user_groups")
 */
class UserGroup implements AccessedUsersAndGroupsInterface,
    Loggable,
    ContainerizedInterface,
    RestrictAccessInterface,
    ExpirationInterface,
    DeepCopyInterface
{
    use AccessedUsersAndGroupsTrait;
    use FSContainerTrait;

    use RestrictAccessTrait;
    use ExpirationTrait;

    use TruncateStringTrait;


    const USER_GROUP_REVIEW_SHOW__LIST = 'list';
    const USER_GROUP_REVIEW_SHOW__THUMBNAIL = 'thumbnail';
    /**
     * @var string
     * @ORM\Column(type="string", options={"default":UserGroup::USER_GROUP_REVIEW_SHOW__LIST})
     */
    protected $defaultReviewView;
    /**
     * @var integer
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;
    /**
     * @var string
     * @ORM\Column(type="string")
     * @Assert\NotBlank()
     */
    private $name;
    /**
     * @var integer
     * @Assert\Type(type="integer")
     * @Assert\Range(min=0)
     * @ORM\Column(type="integer")
     */
    private $recordDuration;
    /**
     * @var \Doctrine\Common\Collections\ArrayCollection | User[]
     * @ORM\OneToMany(targetEntity="User", mappedBy="userGroup")
     * @ORM\OrderBy({"username" = "ASC"})
     */
    private $users;
    /**
     * @ORM\OneToMany(targetEntity="FS\UserBundle\Entity\UserGroupRight", mappedBy="group", orphanRemoval=true, cascade={"all"})
     **/
    private $rights;
    /**
     * @var ArrayCollection | \FS\CamBundle\Entity\Room[]
     * @ORM\ManyToMany(targetEntity="FS\CamBundle\Entity\Room")
     * @ORM\JoinTable(name="users_groups_rooms",
     *      joinColumns={@ORM\JoinColumn(name="user_group_id", referencedColumnName="id", onDelete="CASCADE")},
     *      inverseJoinColumns={@ORM\JoinColumn(name="room_id", referencedColumnName="id", onDelete="CASCADE")}
     *      )
     **/
    private $rooms;
    /**
     * @var Template
     *
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\Template")
     * @ORM\JoinColumn(name="marker_template_id", referencedColumnName="id")
     **/
    private $markerTemplate;
    /**
     * @var Template
     *
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\Template")
     * @ORM\JoinColumn(name="info_template_id", referencedColumnName="id")
     **/
    private $infoTemplate;
    /**
     * @var RetentionRule
     *
     * @ORM\ManyToOne(targetEntity="FS\CamBundle\Entity\RetentionRule",cascade={"persist"})
     * @ORM\JoinColumn(referencedColumnName="id")
     */
    private $defaultRetentionRule;
    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    private $ioRecordName;
    /**
     * @var Object
     *
     * @ORM\Column(type="json_array", nullable=true)
     */
    private $searchResults;
    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean", options={"default":false})
     */
    private $disabledAccess = false;

    public function __construct()
    {
        $this->users = new ArrayCollection();
        $this->rights = new ArrayCollection();
        $this->recordDuration = 0;
        $this->disabledAccess = false;
        $this->defaultReviewView = self::USER_GROUP_REVIEW_SHOW__LIST;
    }

    public static function getReviewViewList()
    {
        return [
            self::USER_GROUP_REVIEW_SHOW__LIST => ucfirst(self::USER_GROUP_REVIEW_SHOW__LIST),
            self::USER_GROUP_REVIEW_SHOW__THUMBNAIL => ucfirst(self::USER_GROUP_REVIEW_SHOW__THUMBNAIL),
        ];
    }

    /**
     * @return string
     */
    public function getTruncatedName()
    {
        return $this->truncateString($this->getName(), 50);
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return array|\Doctrine\Common\Collections\ArrayCollection | User[]
     */
    public function getUsers()
    {
        return $this->users;
    }

    /**
     * @param array|\Doctrine\Common\Collections\ArrayCollection $users
     */
    public function setUsers($users)
    {
        $this->users = $users;

        return $this;
    }

    public function removeRight(UserGroupRight $right)
    {
        $this->rights->removeElement($right);
    }

    public function hasRight($right)
    {
        $rights = $this->getRights()->filter(
            function ($entry) use ($right) {
                return $entry->isGranted($right);
            }
        );

        return !$rights->isEmpty();
    }

    /**
     * @return ArrayCollection | UserGroupRight[]
     */
    public function getRights()
    {
        return $this->rights;
    }

    /**
     * @param UserGroup $rights
     */
    public function setRights($rights)
    {
        $this->rights = $rights;

        return $this;
    }

    public function __toString()
    {
        return $this->getTruncatedName();
    }

    public function getRooms()
    {
        return $this->rooms;
    }

    public function removeRoom(Room $room)
    {
        $this->rooms->removeElement($room);

        return $this;
    }

    /**
     * Get defaultRetentionRule
     *
     * @return RetentionRule
     */
    public function getDefaultRetentionRule()
    {
        $rule = $this->defaultRetentionRule;
        if (is_null($rule)) {
            $rule = new RetentionRule();
            $rule->setType(RetentionRule::KEEP_INFINITELY);
        }

        return $rule;
    }

    /**
     * Set defaultRetentionRule
     *
     * @param RetentionRule $defaultRetentionRule
     * @return UserGroup
     */
    public function setDefaultRetentionRule($defaultRetentionRule)
    {
        $this->defaultRetentionRule = $defaultRetentionRule;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getIoRecordName()
    {
        return $this->ioRecordName;
    }

    /**
     * @param mixed $ioRecordName
     * @return $this
     */
    public function setIoRecordName($ioRecordName)
    {
        $this->ioRecordName = $ioRecordName;
        return $this;
    }

    /**
     * @return int
     */
    public function getRecordDuration()
    {
        return $this->recordDuration;
    }

    public function setRecordDuration($recordDuration)
    {
        $this->recordDuration = $recordDuration;
        return $this;
    }

    public function getObjectName()
    {
        return $this->name;
    }

    public function getGroupTemplateInfo()
    {
        $data = [];

        $data['userGroup'] = $this->getName();
        $data['userGroupId'] = $this->getId();
        $data['template'] = array();
        if ($this->getInfoTemplate() != null) {
            $fields = $this->getInfoTemplate()->getFields();
            foreach ($fields as $field) {
                if ($field->getType() == TemplateField::TYPE_AUDIO)
                    continue;
                if ($field->getEnabled() == true) {
                    $data['template'][] = $field->toApiArray();
                }
            }
        }

        return $data;
    }

    /**
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param integer $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return Template
     */
    public function getInfoTemplate()
    {
        return $this->infoTemplate;
    }

    /**
     * @param Template $infoTemplate
     */
    public function setInfoTemplate($infoTemplate)
    {
        $this->infoTemplate = $infoTemplate;

        return $this;
    }

    public function getTemplateInfo()
    {
        $data = [];

        $fields = $this->getInfoTemplate()->getFields();
        $data['id'] = $this->getInfoTemplate()->getId();
        $data['name'] = $this->getInfoTemplate()->getName();

        foreach ($fields as $field) {
            if ($field->getType() == TemplateField::TYPE_AUDIO)
                continue;
            if ($field->getEnabled() == true) {
                $data['fields'] = $field->toApiArray();
            }
        }
        return $data;
    }

    public function getTemplateMarker()
    {
        $data = [];

        $fields = $this->getMarkerTemplate()->getFields();
        $data['id'] = $this->getMarkerTemplate()->getId();
        $data['name'] = $this->getMarkerTemplate()->getName();

        foreach ($fields as $field) {
            if ($field->getType() == TemplateField::TYPE_AUDIO)
                continue;
            if ($field->getEnabled() == true) {
                $data['fields'] = $field->toApiArray();
            }
        }
        return $data;
    }

    /**
     * @return mixed
     */
    public function getMarkerTemplate()
    {
        return $this->markerTemplate;
    }

    /**
     * @param mixed $markerTemplate
     */
    public function setMarkerTemplate($markerTemplate)
    {
        $this->markerTemplate = $markerTemplate;

        return $this;
    }

    public function getGroupTemplateMarker()
    {
        $data = [];

        $data['userGroup'] = $this->getName();
        $data['userGroupId'] = $this->getId();
        $data['template'] = array();
        if ($this->getMarkerTemplate() != null) {
            $fields = $this->getMarkerTemplate()->getFields();
            foreach ($fields as $field) {
                if ($field->getType() == TemplateField::TYPE_AUDIO)
                    continue;
                if ($field->getEnabled() == true) {
                    $data['template'][] = $field->toApiArray();
                }
            }
        }

        return $data;
    }

    /**
     * @param ContainerizedInterface $entity
     * @return bool
     */
    public function isVisible(ContainerizedInterface $entity)
    {
        $container = $this->getContainer();
        $entityContainer = $entity->getContainer();

        return $container->isVisible($entityContainer);
    }

    /**
     * @return Object
     */
    public function getSearchResults()
    {
        return $this->searchResults;
    }

    /**
     * @param $searchResults
     * @return $this
     */
    public function setSearchResults($searchResults)
    {
        $this->searchResults = $searchResults;
        return $this;
    }

    /**
     * @return bool
     */
    public function getDisabledAccess()
    {
        return $this->disabledAccess;
    }

    /**
     * @param $disabledAccess
     * @return $this
     */
    public function setDisabledAccess($disabledAccess)
    {
        $this->disabledAccess = $disabledAccess;
        return $this;
    }

    /**
     * @return boolean
     */
    public function hasAccessNow()
    {
        return $this->isAccessToday();
    }

    public function isAccessExpired()
    {
        return $this->checkExpiration();
    }

    /**
     * @return UserGroup
     */
    public function makeDeepCopy()
    {
        $result = clone $this;
        $result->users = new ArrayCollection();
        $result->rooms = new ArrayCollection();
        $result->rights = new ArrayCollection();
        foreach ($this->rooms as $room) {
            $result->addRoom($room);
        }
        foreach ($this->rights as $right) {
            $result->addRight(clone $right);
        }

        $result->setRestrictAccess(RestrictAccessBuilder::cloneRestrictAccess($this->getRestrictAccess()));


        if ($this->getExpiration()) {
            $result->setExpiration(clone $this->getExpiration());
        } else {
            $this->setExpiration(new Expiration());
        }


        return $result;
    }

    public function addRoom(Room $room)
    {
        $this->rooms[] = $room;

        return $this;
    }

    public function addRight(UserGroupRight $right)
    {
        $this->rights[] = $right;
        $right->setGroup($this);

        return $this;
    }

    /**
     * @return string
     */
    public function getDefaultReviewView()
    {
        return $this->defaultReviewView;
    }

    /**
     * @param string $defaultReviewView
     * @return $this
     */
    public function setDefaultReviewView($defaultReviewView)
    {
        $this->defaultReviewView = $defaultReviewView;
        return $this;
    }

}
